﻿using System;
using System.Web;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Web.UI;
using System.Collections.Generic;
using Esnek.BLL;
using Esnek.BLL.WebControls;
using Esnek.DAL;
using System.Data;
using SubSonic;
using SubSonic.Repository;
using SubSonic.Query;
using SubSonic.DataProviders;
using System.Text.RegularExpressions;

namespace Esnek.BLL.PartTemplates
{
    [PartTemplate(Name = "Kutu", TemplateName = "BoxWithZones", SortNo = 500, IsDisabled = true)]
    public partial class BoxWithZones : PartWithZones
    {

        [TextEditor(Title = "Başlık :", SortNo = 300, Height = "auto")]
        public string TitleHtml
        {
            get { return GetPropertyValue<string>("TitleHtml"); }
            set { SetPropertyValue<string>("TitleHtml", value); }
        }

        [TextEditor(Title = "Metin :", Height = "auto", SortNo = 400)]
        public string Text
        {
            get { return GetPropertyValue<string>("Text"); }
            set { SetPropertyValue<string>("Text", value); }
        }

        [UrlProperty(Title = "Bağlantı Konumu :", SortNo = 200)]
        public string Link
        {
            get { return GetPropertyValue<string>("Link"); }
            set { SetPropertyValue<string>("Link", value); }
        }

        [ImageUrlProperty(Title = "Resim Konumu :", SortNo = 100)]
        public string ImageUrl
        {
            get { return GetPropertyValue<string>("ImageUrl"); }
            set { SetPropertyValue<string>("ImageUrl", value); }
        }

        [TitledTextBox(Title = "Resim Genişliği :", IsRequired = true, SortNo = 600, DataType = TypeCode.Int32, TabName = TabNames.Other)]
        public int ImageWidth
        {
            get { return GetPropertyValue<int>("ImageWidth", 400); }
            set { SetPropertyValue<int>("ImageWidth", value); }
        }

        [TitledTextBox(Title = "Resim Yüksekliği :", IsRequired = true, SortNo = 700, DataType = TypeCode.Int32, TabName = TabNames.Other)]
        public int ImageHeight
        {
            get { return GetPropertyValue<int>("ImageHeight", 356); }
            set { SetPropertyValue<int>("ImageHeight", value); }
        }

        [ImageUrlProperty(Title = "Maske Resmi :", SortNo = 750, TabName = TabNames.Other)]
        public string MaskImage
        {
            get { return GetPropertyValue<string>("MaskImage"); }
            set { SetPropertyValue<string>("MaskImage", value); }
        }

        [CheckBox(Title = "Fancybox Etkin :", SortNo = 800, TabName = TabNames.Other)]
        public bool FancyboxEnabled
        {
            get { return GetPropertyValue<bool>("FancyboxEnabled"); }
            set { SetPropertyValue<bool>("FancyboxEnabled", value); }
        }

        [TitledTextBox(Title = "Detaylar Butonu :", SortNo = 1750, TabName = TabNames.Other)]
        public string DetailsButton
        {
            get { return GetPropertyValue<string>("DetailsButton"); }
            set { SetPropertyValue<string>("DetailsButton", value); }
        }

        [CheckBox(Title = "Resimleri Kes :", SortNo = 1800, TabName = TabNames.Other)]
        public bool CropPictures
        {
            get { return GetPropertyValue<bool>("CropPictures", true); }
            set { SetPropertyValue<bool>("CropPictures", value); }
        }

        [CheckBox(Title = "Ön Biçimi Kullan :", SortNo = 1900, TabName = TabNames.Other)]
        public bool UseFormatText
        {
            get { return GetPropertyValue<bool>("UseFormatText", true); }
            set { SetPropertyValue<bool>("UseFormatText", value); }
        }

        [CheckBox(Title = "Siyaz Beyaz Efekti :", TabName = TabNames.Other, SortNo = 2000)]
        public bool HoverBlackWhite
        {
            get { return GetPropertyValue<bool>("HoverBlackWhite", false); }
            set { SetPropertyValue<bool>("HoverBlackWhite", value); }
        }

        [DBProperty]
        public override string ZoneID
        {
            get
            {
                string znID = GetPropertyValue<string>("ZoneID");
                return !string.IsNullOrWhiteSpace(znID) ? znID : ContentDataAccess.GetDefaultPartZoneMapping(PartType);
            }
            set { SetPropertyValue<string>("ZoneID", value); }
        }

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            ContentDataAccess DatabaseManager = new ContentDataAccess();

            Zone convertedContainer = DatabaseManager.GetZone(ZoneID, true, true, (Page as Esnek.BLL.Page).UserProfile.ViewMode);

            if (convertedContainer == null) return;

            convertedContainer.Container.Attributes["class"] += " convertedzone box ";

            SortChildZones(convertedContainer);

            MarkAsConvertedZone(convertedContainer);

            phlBoxWithZones.Controls.Add(convertedContainer);

            Zone MaskContainer = null;
            Zone Valigner = null;

            LoadContainers(convertedContainer, ref MaskContainer, ref Valigner);

            if (Valigner != null)
            {

                HtmlGenericControl table = new HtmlGenericControl("table");
                HtmlGenericControl tr = new HtmlGenericControl("tr");
                HtmlGenericControl td = new HtmlGenericControl("td");
                Valigner.Container.Controls.AddAt(0, table);
                table.Controls.Add(tr);
                tr.Controls.Add(td);
                for (int j = 0; j < Valigner.ChildZones.Count; j++)
                {
                    td.Controls.Add(Valigner.ChildZones[j]);
                    Valigner.ChildZones.RemoveAt(j);
                    j--;
                }

                Valigner.Container.Attributes["class"] += " valigner ";
            }

            if (!string.IsNullOrWhiteSpace(MaskImage) && MaskContainer != null)
            {
                string style = MaskContainer.Container.Attributes["style"];
                style = "background:url('" + MaskImage + "');" + style;
                style += "height:" + ImageHeight.ToString() + "px;";
                MaskContainer.Container.Attributes["style"] = style;
            }

            List<Zone> RemovedZones = new List<Zone>();

            FillItem(convertedContainer, this, ref RemovedZones);

            foreach (Zone removed in RemovedZones)
            {
                removed.ParentZone.ChildZones.Remove(removed);
                removed.Container.Parent.Controls.Remove(removed.Container);
            }

        }

        public void FillPropertyBoundZones(Zone zone)
        {
            if (zone.ChildZones != null)
            {
                foreach (Zone zn in zone.ChildZones)
                    FillPropertyBoundZones(zn);
            }

            if (!string.IsNullOrWhiteSpace(zone.PartRole))
            {
                string formatText = CommonMethods.StripHtmlTags(zone.FormatText);
                if (!string.IsNullOrWhiteSpace(formatText))
                {
                    MatchCollection matches = Regex.Matches(formatText, @"\{(.*?)\}");
                    foreach (Match m in matches)
                    {
                        string pName = m.Value.Trim(new char[] { '{', '}' });
                        object value = this.GetType().GetProperty(pName).GetValue(this, null);
                        if (value != null)
                        {
                            string propertyValue = value.ToString();
                            zone.FormatText = zone.FormatText.Replace(m.Value, CommonMethods.StripHtmlTags(propertyValue));
                        }
                        else zone.FormatText = string.Empty;
                    }
                }
            }
        }

        public void LoadDataBindingInformation(List<string> FieldNamesToBind, Zone zone)
        {
            if (zone.ChildZones != null)
            {
                foreach (Zone zn in zone.ChildZones)
                    LoadDataBindingInformation(FieldNamesToBind, zn);
            }

            if (!string.IsNullOrWhiteSpace(zone.PartRole))
            {
                string formatText = CommonMethods.StripHtmlTags(zone.FormatText);
                if (!string.IsNullOrWhiteSpace(formatText))
                {
                    MatchCollection matches = Regex.Matches(formatText, @"\[(.*?)\]");
                    foreach (Match m in matches)
                        FieldNamesToBind.Add(m.Value.Trim(new char[] { '[', ']' }));
                }
            }
        }

        public void FillItem(Zone zone, BoxWithZones itemData, ref List<Zone> RemovedZones)
        {
            if (zone.ChildZones != null)
            {
                for (int i = 0; i < zone.ChildZones.Count; i++)
                {
                    Zone child = zone.ChildZones[i];
                    FillItem(child, itemData, ref RemovedZones);
                }
            }

            if (string.IsNullOrWhiteSpace(zone.PartRole)) return;
            if (string.IsNullOrWhiteSpace(CommonMethods.StripHtmlTags(zone.FormatText))) return;

            /*HtmlDocument formatHtml = new HtmlDocument();
            formatHtml.LoadHtml(zone.FormatText);*/

            string itemLink = itemData.Link;

            /*if (itemData["Path"] != DBNull.Value)
                itemLink = itemData["Path"].ToString();*/

            bool hasValue = false;

            MatchCollection matches = Regex.Matches(zone.FormatText, @"\{(.*?)\}");
            foreach (Match m in matches)
            {
                string mValue = m.Value.Trim(new char[] { '{', '}' });
                object data = itemData.GetType().GetProperty(mValue).GetValue(itemData, null);
                string val = data != DBNull.Value ? data.ToString() : null;
                if (val != null && CommonMethods.StripHtmlTags(val).Trim() != string.Empty)
                {
                    hasValue = true;
                    BindZoneData(zone, m, val, itemLink);
                }
            }
            if (matches.Count == 0 && !string.IsNullOrWhiteSpace(itemLink))
            {
                hasValue = true;
                //BindZoneData(zone, null, null, itemLink);
                zone.Container.InnerHtml = zone.FormatText;
            }

            if (string.IsNullOrWhiteSpace(itemLink) && zone.BlockName == "details") hasValue = false;

            if (!hasValue)
            {
                if (string.IsNullOrWhiteSpace(zone.BlockName))
                {
                    RemovedZones.Add(zone);
                }
                else
                {
                    Zone parent = zone.ParentZone;
                    bool removed = false;
                    while (parent != null)
                    {
                        Zone current = parent;
                        if (current.BlockName == zone.BlockName)
                        {
                            RemovedZones.Add(current);
                            removed = true;
                            break;
                        }
                        parent = current.ParentZone;
                    }
                    if (!removed)
                        RemovedZones.Add(zone);
                }
            }
        }

        public void BindZoneData(Zone zone, Match m, string val, string itemLink)
        {
            if (zone.PartRole == "text")
            {
                if (UseFormatText)
                {
                    string pattern = @"\<br\s*\/?\>\s*?\<\/.*?\>";
                    Regex r = new Regex(pattern, RegexOptions.IgnoreCase);
                    val = r.Replace(val, @"\n");

                    pattern = @"(\<\/p\>|\<\/h1\>|\<\/h2\>|\<\/h3\>|\<\/h4\>|\<\/h5\>|\<\/h6\>|\<\/div\>|\<br.*?\>)";
                    r = new Regex(pattern, RegexOptions.IgnoreCase);
                    val = r.Replace(val, @"\n");
                    val = CommonMethods.StripHtmlTags(val);
                    val = val.Replace(@"\n", "<br />");
                    zone.Container.InnerHtml = zone.FormatText.Replace(m.Value, val);
                }
                else
                    zone.Container.InnerHtml = val;

                if (!string.IsNullOrWhiteSpace(itemLink))
                {
                    HtmlGenericControl alink = new HtmlGenericControl("a");
                    alink.Attributes["class"] = " link ";
                    alink.Attributes["href"] = itemLink;
                    zone.Container.Controls.Add(alink);
                }

            }
            else if (zone.PartRole == "image")
            {
                string replace = null;

                for (int i = 0; i < zone.CssStyles.Controls.Count; i++)
                {
                    CssStyles cssStyle = (CssStyles)zone.CssStyles.Controls[i];
                    if (cssStyle.StyleName == "background")
                    {
                        replace = "background:" + cssStyle.StyleValue;
                    }
                }

                if (replace != null) zone.Container.Attributes["style"] = zone.Container.Attributes["style"].Replace(replace, "");

                /*if (!string.IsNullOrWhiteSpace(ImageUrl))
                {
                    string style = ImageContainer.Container.Attributes["style"];
                    style = "background:url('" + CommonMethods.GetResizedImagePath(ImageUrl, ImageWidth, ImageHeight, CropPictures) + "');" + style;
                    style += "height:" + ImageHeight.ToString() + "px;";
                    ImageContainer.Container.Attributes["style"] = style;
                    ImageContainer.Container.Attributes["class"] += " image-container ";
                }*/

                try
                {
                    if (!HoverBlackWhite)
                    {
                        string resizedImage = CommonMethods.GetResizedImagePath(val, (int)ImageWidth, (int)ImageHeight, CropPictures);
                        HtmlImage img = new HtmlImage();
                        img.Src = resizedImage;
                        img.Attributes["style"] = "position:absolute;top:0;left:0";
                        zone.Container.Controls.Add(img);
                    }
                    else
                    {
                        string resizedImage = CommonMethods.GetResizedImagePath(val, (int)ImageWidth, (int)ImageHeight, CropPictures);
                        HtmlImage img = new HtmlImage();
                        img.Src = resizedImage;
                        img.Attributes["style"] = "position:absolute;top:0;left:0";
                        img.Attributes["class"] += " fade-image-b";
                        zone.Container.Controls.Add(img);

                        img = new HtmlImage();
                        img.Src = CommonMethods.GetResizedImagePath(val, (int)ImageWidth, (int)ImageHeight, CropPictures, true);
                        img.Attributes["style"] = "position:absolute;top:0;left:0";
                        img.Attributes["class"] = "fade-image-a";
                        zone.Container.Controls.Add(img);
                        zone.Container.Attributes["class"] += " fade-image ";
                    }

                    if (!string.IsNullOrWhiteSpace(itemLink))
                    {
                        HtmlGenericControl alink = new HtmlGenericControl("a");
                        alink.Attributes["class"] = " link ";
                        alink.Attributes["href"] = itemLink;
                        zone.Container.Controls.Add(alink);
                    }
                    else if (FancyboxEnabled)
                    {
                        HtmlGenericControl alink = new HtmlGenericControl("a");
                        alink.Attributes["href"] = val;
                        alink.Attributes["class"] = " link ";
                        alink.Attributes["rel"] = "fancy" + ID;
                        zone.Container.Controls.Add(alink);
                    }

                }
                catch { }

                zone.Container.Attributes["class"] += " image-container ";
            }
            /*else if (zone.PartRole == "date")
            {
                string pattern = @"\<br\s*\/?\>\s*?\<\/.*?\>";
                Regex r = new Regex(pattern, RegexOptions.IgnoreCase);
                val = r.Replace(val, @"\n");

                pattern = @"(\<\/p\>|\<\/h1\>|\<\/h2\>|\<\/h3\>|\<\/h4\>|\<\/h5\>|\<\/h6\>|\<\/div\>|\<br.*?\>)";
                r = new Regex(pattern, RegexOptions.IgnoreCase);
                val = r.Replace(val, @"\n");
                val = CommonMethods.StripHtmlTags(val);
                val = val.Replace(@"\n", "<br />");
                zone.Container.InnerHtml = zone.FormatText.Replace(m.Value, Convert.ToDateTime(val).ToString(DateFormat));
            }*/
            else if (zone.PartRole == "link")
            {
                HtmlAnchor link = new HtmlAnchor();
                link.HRef = val.EndsWith(".aspx") ? val : val + ".aspx";
                zone.Container.Controls.Add(link);
                zone.Container.Attributes["class"] += " link ";
                link.Attributes["class"] += " link ";
            }

        }

        public void SortChildZones(Zone zn)
        {
            if (zn.ChildZones != null)
            {
                foreach (Zone child in zn.ChildZones)
                    SortChildZones(child);

                zn.ChildZones.Sort(delegate(Zone a, Zone b)
                {
                    return a.SortNo.CompareTo(b.SortNo);
                });
            }

        }

        public void MarkAsConvertedZone(Zone zn)
        {
            MarkAsConvertedZone(zn, true);
        }

        public void MarkAsConvertedZone(Zone zn, bool deep)
        {
            if (deep && zn.ChildZones != null)
            {
                foreach (Zone child in zn.ChildZones)
                    MarkAsConvertedZone(child, deep);
            }

            string classname = " convertedzone ";
            if (zn.HasClft) classname += "clft ";

            zn.Container.Attributes["class"] += classname;

        }

        public void LoadContainer(Zone container, ref Zone ItemContainer, string partRole)
        {
            LoadContainer(container, ref ItemContainer, partRole, null);
        }

        public void LoadContainer(Zone container, ref Zone ItemContainer, string partRole, HashSet<Zone> not)
        {
            if (container.ChildZones == null) return;

            foreach (Zone zn in container.ChildZones)
            {
                if (not != null)
                {
                    if (not.Contains(zn)) continue;
                }

                if (zn.PartRole == partRole)
                {
                    ItemContainer = zn;
                    break;
                }

                if (zn.ChildZones != null && ItemContainer == null)
                {
                    LoadContainer(zn, ref ItemContainer, partRole, not);
                }
            }
        }

        public void LoadContainers(Zone container, ref Zone MaskContainer, ref Zone Valigner)
        {
            foreach (Zone zn in container.ChildZones)
            {

                if (zn.PartRole == "mask")
                {
                    if (MaskContainer == null)
                        MaskContainer = zn;
                }
                else if (zn.PartRole == "valigner")
                {
                    if (Valigner == null)
                        Valigner = zn;
                }

                if (zn.ChildZones != null)
                {
                    LoadContainers(zn, ref MaskContainer, ref Valigner);
                }
            }
        }

    }
}
