﻿using System;
using System.Web;
using Esnek.BLL;
using Esnek.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using System.Text.RegularExpressions;


namespace Esnek.BLL.PartTemplates
{
    [PartTemplate(Name = "Resim Kutusu", TemplateName = "BoxedImage", SortNo = 200)]
    public partial class BoxedImage : Esnek.BLL.Part
    {

        [TextEditor(Title = "Başlık :", SortNo = 100, Height="auto")]
        public string TitleHtml
        {
            get { return GetPropertyValue<string>("TitleHtml"); }
            set { SetPropertyValue<string>("TitleHtml", value); }
        }

        [ImageUrlProperty(Title = "Resim Konumu :", SortNo = 200)]
        public string ImageUrl
        {
            get { return GetPropertyValue<string>("ImageUrl"); }
            set { SetPropertyValue<string>("ImageUrl", value); }
        }

        [UrlProperty(Title = "Bağlantı Konumu :", SortNo = 300)]
        public string TitleUrl
        {
            get { return GetPropertyValue<string>("TitleUrl"); }
            set { SetPropertyValue<string>("TitleUrl", value); }
        }

        [TitledTextBox(Title = "Resim Genişliği :", SortNo = 500)]
        public string ImageWidth
        {
            get { return GetPropertyValue<string>("ImageWidth"); }
            set { SetPropertyValue<string>("ImageWidth", value); }
        }

        [TitledTextBox(Title = "Resim Yüksekliği :", SortNo = 600)]
        public string ImageHeight
        {
            get { return GetPropertyValue<string>("ImageHeight"); }
            set { SetPropertyValue<string>("ImageHeight", value); }
        }

        [CheckBox(Title = "Fancybox Etkin :", SortNo = 700)]
        public bool FancyboxEnabled
        {
            get { return GetPropertyValue<bool>("FancyboxEnabled"); }
            set { SetPropertyValue<bool>("FancyboxEnabled", value); }
        }

        [CheckBox(Title = "Başlık Altta :", SortNo = 800)]
        public bool ImageAlignLeft
        {
            get { return GetPropertyValue<bool>("ImageAlignLeft"); }
            set { SetPropertyValue<bool>("ImageAlignLeft", value); }
        }

        public int parseInt(string str)
        {
            if (str == null) return 0;
            foreach (Match match in Regex.Matches(str, @"^\d+"))
            {
                return int.Parse(match.Value);
            }
            return 0;
        }

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            HtmlGenericControl partContent = new HtmlGenericControl("div");
            partContent.Attributes["class"] = "part_content";
            HtmlGenericControl partHeader = null;

            if (ImageAlignLeft) phlBoxedImage.Controls.Add(partContent);

            if (TitleHtml != null && CommonMethods.StripHtmlTags(TitleHtml).Trim() != string.Empty)
            {
                partHeader = new HtmlGenericControl("div");
                partHeader.Attributes["class"] = "part_header";
                partContent.Attributes["class"] += " with-header";                

                Literal titleHtml = new Literal();
                titleHtml.Text = TitleHtml;
                partHeader.Controls.Add(titleHtml);
                phlBoxedImage.Controls.Add(partHeader);
            }

            if (!ImageAlignLeft) phlBoxedImage.Controls.Add(partContent);

            HtmlGenericControl anc = new HtmlGenericControl("a");
            HtmlImage img = new HtmlImage();
            if (ImageUrl != null && ImageUrl != string.Empty)
            {
                int imgW = parseInt(ImageWidth);
                int imgH = parseInt(ImageHeight);

                if ((imgW > 0 || imgH > 0) && !ImageUrl.ToLower().StartsWith("http"))
                {                    
                    img.Src = CommonMethods.GetResizedImagePath(ImageUrl, imgW, imgH, false);
                }
                else img.Src = ImageUrl;

                if (imgW > 0) img.Width = imgW;
                if (imgH > 0) img.Height = imgH;


                img.Attributes["onload"] = "$(this).height($(this).height())";
            }

            if (TitleUrl != null && TitleUrl != string.Empty)
            {
                anc.Attributes["href"] = TitleUrl;
            }
            else if (FancyboxEnabled)
            {
                anc.Attributes["href"] = ImageUrl;
                anc.Attributes["rel"] = "fancy" + (Page as Esnek.BLL.Page).ID;
            }

            anc.Controls.Add(img);

            partContent.Controls.Add(anc);

        }

    }
}