﻿using System;
using System.Web;
using System.Data;
using Esnek.BLL;
using Esnek.Web.UI;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using Esnek.BLL;
using Esnek.DAL;

namespace Esnek.BLL.PartTemplates
{
    [PartTemplate(Name = "Breadcrumbs", TemplateName = "BreadcrumbsWithZones", SortNo = 100, IsDisabled = true)]
    public partial class BreadcrumbsWithZones : PartWithZones
    {

        [TitledTextBox(Title = "Başlık :", SortNo = 100)]
        public string Title
        {
            get { return GetPropertyValue<string>("Title"); }
            set { SetPropertyValue<string>("Title", value); }
        }

        [PageUrlProperty(Title = "Kök Sayfa :", IsRequired = true, SortNo= 200)]
        public string RootPage
        {
            get { return GetPropertyValue<string>("RootPage", "/"); }
            set { SetPropertyValue<string>("RootPage", value); }
        }

        [DBProperty]
        public override string ZoneID
        {
            get
            {
                string znID = GetPropertyValue<string>("ZoneID");
                return !string.IsNullOrWhiteSpace(znID) ? znID : ContentDataAccess.GetDefaultPartZoneMapping(PartType);
            }
            set { SetPropertyValue<string>("ZoneID", value); }
        }

        public string CurrentPage { get { return Page != null ? (Page as Esnek.BLL.Page).Url : string.Empty; } }

        ContentDataAccess DatabaseManager = new ContentDataAccess();

        Zone ItemSourceN = null;
        Zone ItemCurrentSourceN = null;
        Zone LevelN = null;

        int MaxLevel = 0;

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            ViewMode ViewMode;
            UserProfile UserProfile = (Page as Page).UserProfile;

            if (UserProfile != null)
                ViewMode = UserProfile.ViewMode;
            else
                ViewMode = ViewMode.Published;

            DataTable Breadcrumbs = DatabaseManager.GetBreadcrumbs(RootPage, CurrentPage, ViewMode);

            if (Breadcrumbs == null) return;


            DataRow[] paths = Breadcrumbs.Select("PropertyName = 'Path'");

            //mevlut bad code
            List<DataRow> sorter = new List<DataRow>(paths);
            sorter.Sort(delegate(DataRow a, DataRow b)
            {
                int sortNo_a = 0;
                int sortNo_b = 0;
                sortNo_a = Breadcrumbs.Select(string.Format(
                    "ObjectID = {0} and PropertyName = 'Path'", a["ObjectID"].ToString()))[0]["Value"].ToString().Length;
                sortNo_b = Breadcrumbs.Select(string.Format(
                    "ObjectID = {0} and PropertyName = 'Path'", b["ObjectID"].ToString()))[0]["Value"].ToString().Length;
                return sortNo_a.CompareTo(sortNo_b);
            });
            paths = sorter.ToArray();
            //bad code end

            Zone convertedContainer = DatabaseManager.GetZone(ZoneID, true, true, (Page as Esnek.BLL.Page).UserProfile.ViewMode);

            if (convertedContainer == null) return;

            Zone ItemContainer = convertedContainer;

            Zone breadcrumbs = CommonMethods.CloneZone(ItemContainer);

            MarkAsConvertedZone(breadcrumbs);

            phlBreadcrumbs.Controls.Add(breadcrumbs);


            int c = 0;

          

                DataRow path = paths[c];

                Zone currentLevel = null;

                LoadItemContainer(breadcrumbs, ref currentLevel, "level-" + c.ToString());

                LevelN = currentLevel;

                if (currentLevel == null) return;

                Zone itemSource = null;
                Zone itemCurrentSource = null;

                LoadItemContainer(currentLevel, ref itemSource, "item-" + c.ToString());
                LoadItemContainer(currentLevel, ref itemCurrentSource, "item-" + c.ToString() + "-current");

                bool IsCurrentPage = Page.ID == path["ObjectID"].ToString();

                bool IsRedirectPage = false;

                bool IsNewWindow = false;

                if (!IsCurrentPage)
                {
                    IsRedirectPage = Breadcrumbs.Select(string.Format(
                    "ObjectID = {0} and PropertyName = 'TemplateName'", path["ObjectID"].ToString()))[0]["Value"].ToString() == "Redirect";

                    if (IsRedirectPage)
                    {
                        DataRow[] selection = Breadcrumbs.Select(string.Format(
                    "ObjectID = {0} and PropertyName = 'NewWindow'", path["ObjectID"].ToString()));

                        if (selection.Length > 0)
                        {
                            try
                            {
                                IsNewWindow = selection[0]["NewWindow"].ToString() == "True";
                            }
                            catch { }
                        }

                        IsCurrentPage = Page.ID == DatabaseManager.GetPageID(Breadcrumbs.Select(string.Format(
                    "ObjectID = {0} and PropertyName = 'RedirectUrl'", path["ObjectID"].ToString()))[0]["Value"].ToString());

                    }
                }

                Zone item = null;

                if (!IsCurrentPage)
                {
                    item = CommonMethods.CloneZone(itemSource);
                }
                else
                {
                    item = CommonMethods.CloneZone(itemCurrentSource);
                }

                MarkAsConvertedZone(item);

                currentLevel.Container.Controls.Add(item);

                Zone ImageContainer = null;
                Zone LinkContainer = null;
                Zone TextContainer = null;

                LoadContainers(item, ref ImageContainer, ref LinkContainer, ref TextContainer);

                HtmlGenericControl anc = new HtmlGenericControl("a");
                anc.Attributes["href"] = path["Value"].ToString() + ".aspx";
                anc.Attributes["class"] += "link ";
                if (IsCurrentPage) anc.Attributes["class"] += "current ";
                if (IsRedirectPage) anc.Attributes["class"] += "redirect ";
                if (IsNewWindow) anc.Attributes["target"] = "_blank";

                Control parent = LinkContainer.Parent;
                int ind = parent.Controls.IndexOf(LinkContainer);
                parent.Controls.AddAt(ind, anc);
                anc.Controls.Add(LinkContainer);

                string pageTitle = Breadcrumbs.Select(string.Format(
                    "ObjectID = {0} and PropertyName = 'Title'", path["ObjectID"].ToString()))[0]["Value"].ToString();

                Literal lt = new Literal();
                lt.Text = pageTitle;
                TextContainer.Container.Controls.Add(lt);

                TextContainer.Container.Attributes["class"] += " text";

                /*DataRow[] menuIcon = Breadcrumbs.Select(string.Format(
                    "ObjectID = {0} and PropertyName = 'MenuIcon'", path["ObjectID"].ToString()));

                if (menuIcon.Length > 0)
                {
                    HtmlImage menuImage = new HtmlImage();
                    menuImage.Src = menuIcon[0]["MenuIcon"].ToString();
                    menuImage.Attributes["class"] = "menu-item-icon";
                    ImageContainer.Container.Controls.Add(menuImage);
                }*/
            
            AppendSubPages(c, item, paths, Breadcrumbs);


                currentLevel.Container.Attributes["class"] += " lvl lvl" + c.ToString();

                if (itemSource != null)
                    itemSource.Parent.Controls.Remove(itemSource);
                if (itemCurrentSource != null)
                    itemCurrentSource.Parent.Controls.Remove(itemCurrentSource);
           


        }

        public void MarkAsConvertedZone(Zone zn)
        {
            if (zn.ChildZones != null)
            {
                foreach (Zone child in zn.ChildZones)
                    MarkAsConvertedZone(child);
            }

            zn.Container.Attributes["class"] += " convertedzone ";
        }

        private void AppendSubPages(int level, Zone Container, DataRow[] paths, DataTable Breadcrumbs)
        {
            level++;

            Zone currentLevel = null;

            LoadItemContainer(Container, ref currentLevel, "level-" + level.ToString());

            if (level >= paths.Length)
            {
                if(currentLevel != null) currentLevel.Parent.Controls.Remove(currentLevel);
                return;
            }

            if (currentLevel == null)
            {
                currentLevel = CommonMethods.CloneZone(LevelN);
                MarkAsConvertedZone(currentLevel);
                Container.Container.Controls.Add(currentLevel);
                if (MaxLevel == 0) MaxLevel = level;
            }
            else LevelN = currentLevel;

            Zone itemSource = null;
            Zone itemCurrentSource = null;

            string checkLevel = MaxLevel == 0 ? level.ToString() : (MaxLevel - 1).ToString();

            LoadItemContainer(currentLevel, ref itemSource, "item-" + checkLevel);
            LoadItemContainer(currentLevel, ref itemCurrentSource, "item-" + checkLevel + "-current");

            /*if (SubPages.Length > 0)
            {*/

                    DataRow path = paths[level];

                    bool IsCurrentPage = Page.ID == path["ObjectID"].ToString();

                    bool IsRedirectPage = false;

                    bool IsNewWindow = false;

                    if (!IsCurrentPage)
                    {
                        IsRedirectPage = Breadcrumbs.Select(string.Format(
                        "ObjectID = {0} and PropertyName = 'TemplateName'", path["ObjectID"].ToString()))[0]["Value"].ToString() == "Redirect";

                        if (IsRedirectPage)
                        {
                            DataRow[] selection = Breadcrumbs.Select(string.Format(
                        "ObjectID = {0} and PropertyName = 'NewWindow'", path["ObjectID"].ToString()));

                            if (selection.Length > 0)
                                IsNewWindow = selection[0]["NewWindow"].ToString() == "True";

                            IsCurrentPage = Page.ID == DatabaseManager.GetPageID(Breadcrumbs.Select(string.Format(
                        "ObjectID = {0} and PropertyName = 'RedirectUrl'", path["ObjectID"].ToString()))[0]["Value"].ToString());

                        }
                    }

                    Zone item = null;

                    if (!IsCurrentPage)
                    {
                        item = CommonMethods.CloneZone(itemSource);
                    }
                    else
                    {
                        item = CommonMethods.CloneZone(itemCurrentSource);
                    }

                    MarkAsConvertedZone(item);
                    currentLevel.Container.Controls.Add(item);

                    Zone ImageContainer = null;
                    Zone LinkContainer = null;
                    Zone TextContainer = null;

                    LoadContainers(item, ref ImageContainer, ref LinkContainer, ref TextContainer);

                    HtmlGenericControl anc = new HtmlGenericControl("a");
                    anc.Attributes["href"] = path["Value"].ToString() + ".aspx";
                    anc.Attributes["class"] += "link ";
                    if (IsCurrentPage) anc.Attributes["class"] += "current ";
                    if (IsRedirectPage) anc.Attributes["class"] += "redirect ";
                    if (IsNewWindow) anc.Attributes["target"] = "_blank";

                    Control parent = LinkContainer.Parent;
                    int ind = parent.Controls.IndexOf(LinkContainer);
                    parent.Controls.AddAt(ind, anc);
                    anc.Controls.Add(LinkContainer);

                    string pageTitle = Breadcrumbs.Select(string.Format(
                            "ObjectID = {0} and PropertyName = 'Title'", path["ObjectID"].ToString()))[0]["Value"].ToString();

                    Literal lt = new Literal();
                    lt.Text = pageTitle;
                    TextContainer.Container.Controls.Add(lt);
                  
                    TextContainer.Container.Attributes["class"] += " text";

                    /*DataRow[] menuIcon = Breadcrumbs.Select(string.Format(
                        "ObjectID = {0} and PropertyName = 'MenuIcon'", path["ObjectID"].ToString()));

                    if (menuIcon.Length > 0)
                    {
                        HtmlImage menuImage = new HtmlImage();
                        menuImage.Src = menuIcon[0]["MenuIcon"].ToString();
                        menuImage.Attributes["class"] = "menu-item-icon";
                        ImageContainer.Container.Controls.Add(menuImage);
                    }*/

                    AppendSubPages(level, item, paths, Breadcrumbs);

                currentLevel.Container.Attributes["class"] += " lvl lvl" + level.ToString();
            /*}
            else
            {
                currentLevel.Parent.Controls.Remove(currentLevel);
            }*/


            if (itemSource != null)
                itemSource.Parent.Controls.Remove(itemSource);
            if (itemCurrentSource != null)
                itemCurrentSource.Parent.Controls.Remove(itemCurrentSource);
        }

        public void LoadContainers(Zone container, ref Zone ImageContainer, ref Zone LinkContainer, ref Zone TextContainer)
        {
            foreach (Zone zn in container.ChildZones)
            {
                //zn.Container.Attributes["class"] += " convertedzone ";

                if (zn.PartRole == "image")
                {
                    if (ImageContainer == null)
                        ImageContainer = zn;
                }
                else if (zn.PartRole == "link")
                {
                    if (LinkContainer == null)
                        LinkContainer = zn;
                }
                else if (zn.PartRole == "text")
                {
                    if (TextContainer == null)
                        TextContainer = zn;
                }

                if (zn.ChildZones != null)
                {
                    LoadContainers(zn, ref ImageContainer, ref LinkContainer, ref TextContainer);
                }
            }
        }

        public void LoadItemContainer(Zone container, ref Zone ItemContainer, string levelName)
        {
            if (container.ChildZones == null) return;

            foreach (Zone zn in container.ChildZones)
            {
                if (zn.PartRole == levelName)
                {
                    ItemContainer = zn;
                    break;
                }

                if (zn.ChildZones != null && ItemContainer == null)
                {
                    LoadItemContainer(zn, ref ItemContainer, levelName);
                }
            }
        }


    }
}