using System;
using System.Web;
using System.Data;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using SubSonic.Repository;
using SubSonic.DataProviders;
using SubSonic.Query;
using HtmlAgilityPack;
using System.Text.RegularExpressions;
using Esnek.BLL;
using Esnek.DAL;
using System.Net.Mail;
using System.Text;

namespace Esnek.BLL.PartTemplates
{

    [PartTemplate(Name = "Form", TemplateName = "CustomFormWithZones", SortNo = 400)]
    public partial class CustomFormWithZones : PartWithZones
    {

        [TextEditor(Title = "Başlık :", IsRequired = true, SortNo = 100, Height = "auto")]
        public string TitleHtml
        {
            get { return GetPropertyValue<string>("TitleHtml"); }
            set { SetPropertyValue<string>("TitleHtml", value); }
        }

        [CheckBox(Title = "Başlık Görünsün :", SortNo = 200)]
        public bool ShowTitle
        {
            get { return GetPropertyValue<bool>("ShowTitle", true); }
            set { SetPropertyValue<bool>("ShowTitle", value); }
        }

        [TitledTextBox(Title = "Gönderilecek E-mail :", IsRequired = true, SortNo = 300)]
        public string FormTo
        {
            get { return GetPropertyValue<string>("FormTo"); }
            set { SetPropertyValue<string>("FormTo", value); }
        }

        [TitledTextBox(Title = "Gerekli Alan Hata :", IsRequired = true, SortNo = 1200)]
        public string FormRequiredError
        {
            get { return GetPropertyValue<string>("FormRequiredError", "Mesajınız iletilemedi! Lütfen gerekli alanları doldurup tekrar deneyiniz."); }
            set { SetPropertyValue<string>("FormRequiredError", value); }
        }

        [TitledTextBox(Title = "Mesaj Gönderildi :", IsRequired = true, SortNo = 1300)]
        public string FormMessageSent
        {
            get { return GetPropertyValue<string>("FormMessageSent", "Mesajınız başarıyla iletildi!"); }
            set { SetPropertyValue<string>("FormMessageSent", value); }
        }

        [TitledTextBox(Title = "Mesaj Hata :", IsRequired = true, SortNo = 1400)]
        public string FormMessageError
        {
            get { return GetPropertyValue<string>("FormMessageError", "Hata! Mesajınız İletilemedi"); }
            set { SetPropertyValue<string>("FormMessageError", value); }
        }

        [TitledTextBox(Title = "Gerekli Alan :", IsRequired = true, SortNo = 1500)]
        public string FormRequired
        {
            get { return GetPropertyValue<string>("FormRequired", "gerekli."); }
            set { SetPropertyValue<string>("FormRequired", value); }
        }

        [TitledTextBox(Title = "Gönder :", IsRequired = true, SortNo = 1600)]
        public string FormSend
        {
            get { return GetPropertyValue<string>("FormSend", "Gönder"); }
            set { SetPropertyValue<string>("FormSend", value); }
        }

        [TitledTextBox(Title = "Gönderen :", SortNo = 1700, AuthorizedRoles = new string[] { "Super User" })]
        public string FormSender
        {
            get {

                string formSender = GetPropertyValue<string>("FormSender");
                if (string.IsNullOrWhiteSpace(formSender))
                {
                    Site esnSite = (HttpContext.Current.Items["esnCurrentSite"] as Site);
                    if (esnSite == null) esnSite = new Esnek.DAL.ContentDataAccess().GetSiteByID("1");
                    if (esnSite != null) formSender = esnSite.SmtpAccount;
                    else formSender = string.Empty;
                }
                return formSender;
            }
            set { SetPropertyValue<string>("FormSender", value); }
        }

        [ControlFactory(Title = "Form Alanları", OutputTypes = new string[] { ControlFactoryOutputTypes.FormInputV3 },
            CssClass = "factory-gallery", TabName = TabNames.FormFields)]
        public ControlFactory FormFields
        {
            get { return GetPropertyValue<ControlFactory>("FormFields"); }
            set { SetPropertyValue<ControlFactory>("FormFields", value); }
        }

        [DBProperty]
        public override string ZoneID
        {
            get
            {
                string znID = GetPropertyValue<string>("ZoneID");
                return !string.IsNullOrWhiteSpace(znID) ? znID : ContentDataAccess.GetDefaultPartZoneMapping(PartType);
            }
            set { SetPropertyValue<string>("ZoneID", value); }
        }

        Button FormSendButton;
        HtmlGenericControl ResultMessage = new HtmlGenericControl();

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            if (!Page.ClientScript.IsClientScriptIncludeRegistered("formjs"))
            {
                Page.ClientScript.RegisterClientScriptInclude(Page.GetType(), "formjs", "/sites/default/js/jquery.esnekform.js");
            }

            ContentDataAccess DatabaseManager = new ContentDataAccess();

            Zone convertedContainer = DatabaseManager.GetZone(ZoneID, true, true, ViewMode.Preview);// (Page as Esnek.BLL.Page).UserProfile.ViewMode);

            if (convertedContainer == null) return;

            convertedContainer.MinHeight.Value = "min-height:0px";

            convertedContainer.Container.Attributes["class"] += " customform ";

            FillPropertyBoundZones(convertedContainer);

            SortChildZones(convertedContainer);

            MarkAsConvertedZone(convertedContainer);

            Zone FieldContainer = null;

            LoadContainer(convertedContainer, ref FieldContainer, "form-fields");

            if (FieldContainer == null) return;

            ResultMessage.Visible = false;

            Zone ResultMessageContainer = null;
            LoadContainer(convertedContainer, ref ResultMessageContainer, "result-message");

            if (ResultMessageContainer != null)
            {
                ResultMessageContainer.MinHeight.Value = "min-height:0px";
                if (!string.IsNullOrWhiteSpace(ResultMessageContainer.FormatText))
                {
                    HtmlDocument messageFormat = new HtmlDocument();
                    messageFormat.LoadHtml(ResultMessageContainer.FormatText);
                    ResultMessage.Attributes["style"] = HttpUtility.HtmlDecode(messageFormat.DocumentNode.FirstChild.Attributes["style"].Value);
                    ResultMessageContainer.Container.Controls.Add(ResultMessage);
                }
                else ResultMessageContainer.Container.Controls.Add(ResultMessage);
            }
            else
            {
                ResultMessage.Attributes["class"] = "result-message";
                FieldContainer.Container.Controls.AddAt(0, ResultMessage);
            }

            Zone TextInputTemplate = null;
            Zone TextareaTemplate = null;
            Zone FileInputTemplate = null;

            Zone ButtonTemplate = null;
            Zone ButtonItem = null;
            Zone ButtonItemActive = null;

            LoadContainers(convertedContainer, ref TextInputTemplate, ref TextareaTemplate, ref FileInputTemplate, ref ButtonTemplate, ref ButtonItem, ref ButtonItemActive);

            int minH1 = 0;
            int minH2 = 0;

            int minH = Math.Max(minH1, minH2);

            if (ButtonItem != null)
            {
                RemoveStyle(ButtonItem, "margin*");
                ButtonItem.Container.Attributes["class"] += " button-item ";
                int.TryParse(ButtonItem.MinHeight.Value.Split(':')[1], out minH1);
            }

            if (ButtonItemActive != null)
            {
                RemoveStyle(ButtonItemActive, "margin*");
                ButtonItemActive.Container.Attributes["class"] += " button-item-active ";
                int.TryParse(ButtonItemActive.MinHeight.Value.Split(':')[1], out minH2);
            }

            if(ButtonTemplate != null)
                ButtonTemplate.MinHeight.Value = "min-height:" + minH.ToString();


            if (FormFields != null)
            {
                for (int i = 0; i < FormFields.Controls.Count; i++)
                {
                    FormInputV3 input = (FormInputV3)FormFields.Controls[i];

                    Zone FormFieldTemplate = null;
                    if (input.FormInputType == "text")
                    {
                        if (TextInputTemplate == null) continue;
                        FormFieldTemplate = CommonMethods.CloneZone(TextInputTemplate);
                    }
                    else if (input.FormInputType == "textarea")
                    {
                        if (TextareaTemplate == null) continue;
                        FormFieldTemplate = CommonMethods.CloneZone(TextareaTemplate);
                    }
                    else if (input.FormInputType == "upload")
                    {
                        if (FileInputTemplate == null) continue;
                        FormFieldTemplate = CommonMethods.CloneZone(FileInputTemplate);
                    }
                    else continue;

                    MarkAsConvertedZone(FormFieldTemplate);

                    Zone FieldItem = null;
                    Zone FieldItemActive = null;

                    LoadContainer(FormFieldTemplate, ref FieldItem, "field-item");
                    LoadContainer(FormFieldTemplate, ref FieldItemActive, "field-item-active");

                    minH1 = 0;
                    minH2 = 0;

                    if (FieldItem != null)
                    {
                        RemoveStyle(FieldItem, "margin*");
                        FieldItem.Container.Attributes["class"] += " field-item ";
                        int.TryParse(FieldItem.MinHeight.Value.Split(':')[1], out minH1);
                    }                    

                    if (FieldItemActive != null)
                    {
                        RemoveStyle(FieldItemActive, "margin*");
                        FieldItemActive.Container.Attributes["class"] += " field-item-active ";
                        int.TryParse(FieldItemActive.MinHeight.Value.Split(':')[1], out minH2);
                    }

                    minH = Math.Max(minH1, minH2);

                    FormFieldTemplate.MinHeight.Value = "min-height:" + minH.ToString();

                    if (FieldItem == null && FieldItemActive == null) continue;

                    List<Zone> RemovedZones = new List<Zone>();

                    if (FieldItem != null)
                    {
                        Zone ValidatorContainer = null;
                        LoadContainer(FieldItem, ref ValidatorContainer, "required-message");
                        FillItem(FieldItem, input, ref RemovedZones, ref ValidatorContainer, false);
                    }

                    if (FieldItemActive != null)
                    {
                        Zone ValidatorContainer = null;
                        LoadContainer(FieldItemActive, ref ValidatorContainer, "required-message");
                        FillItem(FieldItemActive, input, ref RemovedZones, ref ValidatorContainer, true);
                    }

                    foreach (Zone removed in RemovedZones)
                    {
                        removed.ParentZone.ChildZones.Remove(removed);
                        removed.Container.Parent.Controls.Remove(removed.Container);
                    }

                    FieldContainer.Container.Controls.Add(FormFieldTemplate);
                }
            }            

            if (TextInputTemplate != null)
                TextInputTemplate.ParentZone.ChildZones.Remove(TextInputTemplate);
            if (TextareaTemplate != null)
                TextareaTemplate.ParentZone.ChildZones.Remove(TextareaTemplate);
            if (FileInputTemplate != null)
                FileInputTemplate.ParentZone.ChildZones.Remove(FileInputTemplate);


            phlCustomFormWithZones.Controls.Add(convertedContainer);

        }

        public void FillPropertyBoundZones(Zone zone)
        {
            if (zone.ChildZones != null)
            {
                foreach (Zone zn in zone.ChildZones)
                    FillPropertyBoundZones(zn);
            }

            if (!string.IsNullOrWhiteSpace(zone.PartRole))
            {
                string formatText = CommonMethods.StripHtmlTags(zone.FormatText);
                if (!string.IsNullOrWhiteSpace(formatText))
                {
                    MatchCollection matches = Regex.Matches(formatText, @"\{(.*?)\}");
                    foreach (Match m in matches)
                    {
                        string pName = m.Value.Trim(new char[] { '{', '}' });
                        object value = this.GetType().GetProperty(pName).GetValue(this, null);
                        if (value != null)
                        {
                            string propertyValue = value.ToString();
                            zone.FormatText = zone.FormatText.Replace(m.Value, CommonMethods.StripHtmlTags(propertyValue));
                        }
                        else zone.FormatText = string.Empty;

                        zone.Container.InnerHtml = zone.FormatText;
                    }
                }

                if (zone.PartRole == "button")
                {
                    FormSendButton = new Button();
                    FormSendButton.CssClass = "form-button";
                    FormSendButton.Click += Form_OnClick;
                    zone.Container.Controls.Add(FormSendButton);
                    zone.Container.Attributes["class"] += " button-container ";
                }
                else if (zone.PartRole == "button-item-active")
                {
                    zone.Container.Attributes["class"] += " button-item-active ";
                }
            }
        }

        public void LoadDataBindingInformation(List<string> FieldNamesToBind, Zone zone)
        {
            if (zone.ChildZones != null)
            {
                foreach (Zone zn in zone.ChildZones)
                    LoadDataBindingInformation(FieldNamesToBind, zn);
            }

            if (!string.IsNullOrWhiteSpace(zone.PartRole))
            {
                string formatText = CommonMethods.StripHtmlTags(zone.FormatText);
                if (!string.IsNullOrWhiteSpace(formatText))
                {
                    MatchCollection matches = Regex.Matches(formatText, @"\[(.*?)\]");
                    foreach (Match m in matches)
                        FieldNamesToBind.Add(m.Value.Trim(new char[] { '[', ']' }));
                }
            }
        }

        public int ctr = 0;

        public void FillItem(Zone zone, FormInputV3 itemData, ref List<Zone> RemovedZones, ref Zone ValidatorContainer, bool isActiveState)
        {
            if (zone.ChildZones != null)
            {
                for (int i = 0; i < zone.ChildZones.Count; i++)
                {
                    Zone child = zone.ChildZones[i];
                    FillItem(child, itemData, ref RemovedZones, ref ValidatorContainer, isActiveState);
                }
            }

            if (string.IsNullOrWhiteSpace(zone.PartRole)) return;

            string itemLink = null;

            bool hasValue = false;

            if (zone.FormatText != null)
            {
                MatchCollection matches = Regex.Matches(zone.FormatText, @"\[(.*?)\]");
                foreach (Match m in matches)
                {
                    string mValue = m.Value.Trim(new char[] { '[', ']' });
                    object data = itemData.GetType().GetProperty(mValue).GetValue(itemData, null);
                    string val = data != DBNull.Value ? data.ToString() : null;
                    if (val != null && CommonMethods.StripHtmlTags(val).Trim() != string.Empty)
                    {
                        hasValue = true;
                        BindZoneData(zone, m, val, itemLink);
                    }
                }
            }

            if (zone.PartRole == "input-control" && itemData.FormInputType == "text")
            {
                zone.Container.Attributes["class"] += " item-input ";
                
                TextBox inputControl = new TextBox();
                inputControl.Attributes["class"] = " input-control ";
                zone.Container.Controls.Add(inputControl);

                inputControl.ID = itemData.ID + (isActiveState ? "-active" : "");
                inputControl.Attributes["data-inputtitle"] = itemData.InputTitle;                            

                HtmlDocument formatHtml = new HtmlDocument();
                if (zone.FormatText != null)
                {
                    formatHtml.LoadHtml(zone.FormatText);
                    inputControl.Attributes["style"] = formatHtml.DocumentNode.FirstChild.Attributes["style"].Value;
                }

                if (itemData.IsRequiredField)
                {
                    RequiredFieldValidator rfv = new RequiredFieldValidator();
                    rfv.ID = "fv" + (isActiveState ? "-active" : "") + inputControl.ID;
                    rfv.ControlToValidate = inputControl.ID;
                    rfv.ValidationGroup = "CustomForm" + this.ID;
                    rfv.EnableClientScript = false;
                    rfv.ForeColor = System.Drawing.Color.Empty;
                    rfv.Display = ValidatorDisplay.Dynamic;
                    rfv.ErrorMessage = itemData.InputTitle + " " + FormRequired;
                    if (ValidatorContainer == null)
                    {
                        rfv.CssClass = "error";
                        zone.Container.Controls.Add(rfv);
                    }
                    else
                    {
                        ValidatorContainer.MinHeight.Value = "min-height:0px";
                        if (!string.IsNullOrWhiteSpace(ValidatorContainer.FormatText))
                        {
                            HtmlDocument validatorFormat = new HtmlDocument();
                            validatorFormat.LoadHtml(ValidatorContainer.FormatText);
                            HtmlGenericControl rfvContainer = new HtmlGenericControl("div");
                            rfvContainer.Attributes["style"] = HttpUtility.HtmlDecode(validatorFormat.DocumentNode.FirstChild.Attributes["style"].Value);
                            ValidatorContainer.Container.Controls.Add(rfvContainer);
                            rfvContainer.Controls.Add(rfv);
                        } 
                        else ValidatorContainer.Container.Controls.Add(rfv);
                    }
                }
                else if (ValidatorContainer != null)
                {
                     ValidatorContainer.ParentZone.ChildZones.Remove(ValidatorContainer);                    
                }
            }
            else if (zone.PartRole == "input-control" && itemData.FormInputType == "textarea")
            {
                zone.Container.Attributes["class"] += " item-input ";

                TextBox inputControl = new TextBox();
                inputControl.Attributes["class"] = " input-control ";
                inputControl.TextMode = TextBoxMode.MultiLine;
                zone.Container.Controls.Add(inputControl);

                inputControl.ID = itemData.ID + (isActiveState ? "-active" : "");
                inputControl.Attributes["data-inputtitle"] = itemData.InputTitle;

                HtmlDocument formatHtml = new HtmlDocument();
                if (zone.FormatText != null)
                {
                    formatHtml.LoadHtml(zone.FormatText);
                    inputControl.Attributes["style"] = formatHtml.DocumentNode.FirstChild.Attributes["style"].Value;
                    inputControl.Attributes["style"] += "height:" + zone.MinHeight.Value.Split(':')[1] + ";";
                }

                if (itemData.IsRequiredField)
                {
                    RequiredFieldValidator rfv = new RequiredFieldValidator();
                    rfv.ID = "fv" + (isActiveState ? "-active" : "") + inputControl.ID;
                    rfv.ControlToValidate = inputControl.ID;
                    rfv.ValidationGroup = "CustomForm" + this.ID;
                    rfv.EnableClientScript = false;
                    rfv.ForeColor = System.Drawing.Color.Empty;
                    rfv.Display = ValidatorDisplay.Dynamic;
                    rfv.ErrorMessage = itemData.InputTitle + " " + FormRequired;
                    if (ValidatorContainer == null)
                    {
                        rfv.CssClass = "error";
                        zone.Container.Controls.Add(rfv);
                    }
                    else
                    {
                        ValidatorContainer.MinHeight.Value = "min-height:0px";
                        if (!string.IsNullOrWhiteSpace(ValidatorContainer.FormatText))
                        {
                            HtmlDocument validatorFormat = new HtmlDocument();
                            validatorFormat.LoadHtml(ValidatorContainer.FormatText);
                            HtmlGenericControl rfvContainer = new HtmlGenericControl("div");
                            rfvContainer.Attributes["style"] = HttpUtility.HtmlDecode(validatorFormat.DocumentNode.FirstChild.Attributes["style"].Value);
                            ValidatorContainer.Container.Controls.Add(rfvContainer);
                            rfvContainer.Controls.Add(rfv);
                        }
                        else ValidatorContainer.Container.Controls.Add(rfv);
                    }
                }
                else if (ValidatorContainer != null)
                {
                     ValidatorContainer.ParentZone.ChildZones.Remove(ValidatorContainer);                    
                }
            }
            else if (zone.PartRole == "input-control" && itemData.FormInputType == "upload")
            {
                
                zone.Container.Attributes["class"] += " item-input ";

                FileUpload inputControl = new FileUpload();
                inputControl.Attributes["class"] = " input-control ";
                zone.Container.Controls.Add(inputControl);

                inputControl.ID = itemData.ID + (isActiveState ? "-active" : "");
                inputControl.Attributes["data-inputtitle"] = itemData.InputTitle;

                /*HtmlDocument formatHtml = new HtmlDocument();
                if (zone.FormatText != null)
                {
                    formatHtml.LoadHtml(zone.FormatText);
                    inputControl.Attributes["style"] = formatHtml.DocumentNode.FirstChild.Attributes["style"].Value;
                    inputControl.Attributes["style"] += "height:" + zone.MinHeight.Value.Split(':')[1] + ";";
                }*/

                if (itemData.IsRequiredField)
                {
                    RequiredFieldValidator rfv = new RequiredFieldValidator();
                    rfv.ID = "fv" + (isActiveState ? "-active" : "") + inputControl.ID;
                    rfv.ControlToValidate = inputControl.ID;
                    rfv.ValidationGroup = "CustomForm" + this.ID;
                    rfv.EnableClientScript = false;
                    rfv.ForeColor = System.Drawing.Color.Empty;
                    rfv.Display = ValidatorDisplay.Dynamic;
                    rfv.ErrorMessage = itemData.InputTitle + " " + FormRequired;
                    if (ValidatorContainer == null)
                    {
                        rfv.CssClass = "error";
                        zone.Container.Controls.Add(rfv);
                    }
                    else
                    {
                        ValidatorContainer.MinHeight.Value = "min-height:0px";
                        if (!string.IsNullOrWhiteSpace(ValidatorContainer.FormatText))
                        {
                            HtmlDocument validatorFormat = new HtmlDocument();
                            validatorFormat.LoadHtml(ValidatorContainer.FormatText);
                            HtmlGenericControl rfvContainer = new HtmlGenericControl("div");
                            rfvContainer.Attributes["style"] = HttpUtility.HtmlDecode(validatorFormat.DocumentNode.FirstChild.Attributes["style"].Value);
                            ValidatorContainer.Container.Controls.Add(rfvContainer);
                            rfvContainer.Controls.Add(rfv);
                        }
                        else ValidatorContainer.Container.Controls.Add(rfv);
                    }
                }
                else if (ValidatorContainer != null)
                {
                    ValidatorContainer.ParentZone.ChildZones.Remove(ValidatorContainer);
                }
            }
            else if (zone.PartRole == "input-title")
            {
                zone.Container.Attributes["class"] += " item-title ";
            }
            /*if (matches.Count == 0 && !string.IsNullOrWhiteSpace(itemLink))
            {
                hasValue = true;
                //BindZoneData(zone, null, null, itemLink);
                zone.Container.InnerHtml = zone.FormatText;
            }

            if (string.IsNullOrWhiteSpace(itemLink) && zone.BlockName == "details") hasValue = false;

            if (!hasValue)
            {
                if (string.IsNullOrWhiteSpace(zone.BlockName))
                {
                    RemovedZones.Add(zone);
                }
                else
                {
                    Zone parent = zone.ParentZone;
                    bool removed = false;
                    while (parent != null)
                    {
                        Zone current = parent;
                        if (current.BlockName == zone.BlockName)
                        {
                            RemovedZones.Add(current);
                            removed = true;
                            break;
                        }
                        parent = current.ParentZone;
                    }
                    if (!removed)
                        RemovedZones.Add(zone);
                }
            }*/
        }

        public void BindZoneData(Zone zone, Match m, string val, string itemLink)
        {
            if (zone.PartRole == "text" || zone.PartRole == "input-title")
            {
                    string pattern = @"\<br\s*\/?\>\s*?\<\/.*?\>";
                    Regex r = new Regex(pattern, RegexOptions.IgnoreCase);
                    val = r.Replace(val, @"\n");

                    pattern = @"(\<\/p\>|\<\/h1\>|\<\/h2\>|\<\/h3\>|\<\/h4\>|\<\/h5\>|\<\/h6\>|\<\/div\>|\<br.*?\>)";
                    r = new Regex(pattern, RegexOptions.IgnoreCase);
                    val = r.Replace(val, @"\n");
                    val = CommonMethods.StripHtmlTags(val);
                    val = val.Replace(@"\n", "<br />");
                    zone.Container.InnerHtml = zone.FormatText.Replace(m.Value, val);
            }
            else if (zone.PartRole == "input-control")
            {


            }
        }

        public void SortChildZones(Zone zn)
        {
            if (zn.ChildZones != null)
            {
                foreach (Zone child in zn.ChildZones)
                    SortChildZones(child);

                zn.ChildZones.Sort(delegate(Zone a, Zone b)
                {
                    return a.SortNo.CompareTo(b.SortNo);
                });
            }

        }

        public void MarkAsConvertedZone(Zone zn)
        {
            MarkAsConvertedZone(zn, true);
        }

        public void MarkAsConvertedZone(Zone zn, bool deep)
        {
            if (deep && zn.ChildZones != null)
            {
                foreach (Zone child in zn.ChildZones)
                    MarkAsConvertedZone(child, deep);
            }

            string classname = " convertedzone ";
            if (zn.HasClft) classname += "clft ";

            zn.Container.Attributes["class"] += classname;

        }

        public void LoadContainer(Zone container, ref Zone ItemContainer, string partRole)
        {
            LoadContainer(container, ref ItemContainer, partRole, null);
        }

        public void LoadContainer(Zone container, ref Zone ItemContainer, string partRole, HashSet<Zone> not)
        {
            if (container.ChildZones == null) return;

            foreach (Zone zn in container.ChildZones)
            {
                if (not != null)
                {
                    if (not.Contains(zn)) continue;
                }

                if (zn.PartRole == partRole)
                {
                    ItemContainer = zn;
                    break;
                }

                if (zn.ChildZones != null && ItemContainer == null)
                {
                    LoadContainer(zn, ref ItemContainer, partRole, not);
                }
            }
        }

        public void LoadContainers(Zone container, ref Zone TextInputTemplate, ref Zone TextareaTemplate, ref Zone FileInputTemplate, ref Zone ButtonTemplate, ref Zone ButtonItem, ref Zone ButtonItemActive)
        {
            foreach (Zone zn in container.ChildZones)
            {
                if (zn.PartRole == "text-input-template")
                {
                    if (TextInputTemplate == null)
                        TextInputTemplate = zn;
                }
                else if (zn.PartRole == "textarea-template")
                {
                    if (TextareaTemplate == null)
                        TextareaTemplate = zn;
                }

                else if (zn.PartRole == "file-input-template")
                {
                    if (FileInputTemplate == null)
                        FileInputTemplate = zn;
                }
                else if (zn.PartRole == "button-template")
                {
                    if (ButtonTemplate == null)
                        ButtonTemplate = zn;
                }
                else if (zn.PartRole == "button-item")
                {
                    if (ButtonItem == null)
                        ButtonItem = zn;
                }
                else if (zn.PartRole == "button-item-active")
                {
                    if (ButtonItemActive == null)
                        ButtonItemActive = zn;
                }

                if (zn.ChildZones != null && (TextInputTemplate == null || TextareaTemplate == null || FileInputTemplate == null || ButtonTemplate == null || ButtonItem == null || ButtonItemActive == null))
                {
                    LoadContainers(zn, ref TextInputTemplate, ref TextareaTemplate, ref FileInputTemplate, ref ButtonTemplate, ref ButtonItem, ref ButtonItemActive);
                }
            }
        }

        public void RemoveStyle(Zone zone, string styleName)
        {
            if (zone.CssStyles == null) return;

            bool startsWith = false;

            if (styleName.EndsWith("*")) { startsWith = true; styleName = styleName.TrimEnd('*'); }

            for (int i = 0; i < zone.CssStyles.Controls.Count; i++)
            {
                CssStyles cssStyle = (CssStyles)zone.CssStyles.Controls[i];
                if ((!startsWith && cssStyle.StyleName == styleName) || (startsWith && cssStyle.StyleName.StartsWith(styleName)))
                {
                    zone.CssStyles.Controls.RemoveAt(i--);
                }
            }

        }

        StringBuilder formData = new StringBuilder();

        List<FileUpload> uploadedFiles = new List<FileUpload>();

        public void LoadFormData(Control control)
        {
            if (control is TextBox)
            {
                if (!control.ID.EndsWith("-active"))
                {                    
                    TextBox tb = control as TextBox;
                    formData.AppendFormat("<b>{0} :</b> {1}<br />", tb.Attributes["data-inputtitle"], tb.Text);
                }
            }
            else if (control is FileUpload)
            {
                if (!control.ID.EndsWith("-active"))
                {
                    FileUpload fu = control as FileUpload;
                    if (fu.HasFile && fu.PostedFile != null && !string.IsNullOrEmpty(fu.FileName))
                    {
                        formData.AppendFormat("<b>{0} :</b> {1}<br />", fu.Attributes["data-inputtitle"], "Ekte");
                        uploadedFiles.Add(fu);
                    }
                }
            }

            foreach (Control child in control.Controls)
            {
                LoadFormData(child);
            }
        }

        protected void Form_OnClick(object sender, EventArgs e)
        {
            LoadFormData(phlCustomFormWithZones);

            ResultMessage.InnerText = "";
            Page.Validate("CustomForm" + this.ID);
            bool FormIsValid = true;
            for (int i = 0; i < Page.Validators.Count; i++)
            {
                if (Page.Validators[i] is BaseValidator)
                {
                    BaseValidator bv = (BaseValidator)Page.Validators[i];
                    if (bv.ValidationGroup == "CustomForm" + this.ID)
                    {
                        if (bv.ID.StartsWith("fv-active")) continue;

                        if (bv is RequiredFieldValidator && !bv.IsValid)
                        {
                            ResultMessage.InnerText = "Mesajınız iletilemedi! Lütfen gerekli alanları doldurup tekrar deneyiniz.";
                            FormIsValid = false;
                        }
                        else if (bv is CustomValidator)
                        {
                            Control controlToValidate = null;
                            string[] data = bv.ErrorMessage.Split('_');
                            if (data.Length > 1)
                            {
                                controlToValidate = pnlCustomFormWithZones.FindControl(data[0]);
                                bv.ErrorMessage = data[1];
                            }
                            else
                                controlToValidate = pnlCustomFormWithZones.FindControl(bv.ControlToValidate);

                            if (controlToValidate is CheckBoxList)
                            {
                                CheckBoxList cbList = (CheckBoxList)controlToValidate;

                                if (cbList.SelectedItem == null)
                                {
                                    bv.IsValid = false;
                                    ResultMessage.InnerText = "Mesajınız iletilemedi! Lütfen gerekli alanları doldurup tekrar deneyiniz.";
                                    FormIsValid = false;
                                }
                            }
                            else if (controlToValidate is DropDownList)
                            {
                                DropDownList ddList = (DropDownList)controlToValidate;

                                if (ddList.SelectedItem == null || ddList.SelectedValue == "0")
                                {
                                    bv.IsValid = false;
                                    ResultMessage.InnerText = "Mesajınız iletilemedi! Lütfen gerekli alanları doldurup tekrar deneyiniz.";
                                    FormIsValid = false;
                                }
                            }
                            else if (controlToValidate is FileUpload)
                            {
                                FileUpload fu = (FileUpload)controlToValidate;

                                if (!fu.HasFile || fu.PostedFile == null || string.IsNullOrEmpty(fu.FileName))
                                {
                                    bv.IsValid = false;
                                    ResultMessage.InnerText = "Mesajınız iletilemedi! Lütfen gerekli alanları doldurup tekrar deneyiniz.";
                                    FormIsValid = false;
                                }
                            }
                        }
                    }
                }
            }

            if (!FormIsValid)
            {
                ResultMessage.Visible = true;
                return;
            }

            List<Attachment> mailAttachments = new List<Attachment>();
            int maxFileSize = 15360 * 1024;
            int totalFileSize = 0;

            foreach (FileUpload fu in uploadedFiles)
            {
                if (fu.HasFile && fu.PostedFile != null && !string.IsNullOrEmpty(fu.FileName))
                {
                    mailAttachments.Add(new Attachment(fu.FileContent, fu.FileName, fu.PostedFile.ContentType));
                    totalFileSize += fu.FileBytes.Length;
                }
            }

            //StringBuilder sb = new StringBuilder();

            /*foreach (Control hgc in phlCustomFormWithZones.Controls)
            {
                if (hgc is HtmlGenericControl)
                {
                    foreach (Control formControl in hgc.Controls)
                    {
                        if (formControl is Label)
                        {
                            Label lbl = (Label)formControl;
                            Control associatedControl = phlCustomFormWithZones.FindControl(lbl.AssociatedControlID);
                            if (associatedControl is TextBox)
                            {
                                sb.AppendFormat("<b>{0} :</b> {1}<br />", lbl.Text, (associatedControl as TextBox).Text);
                            }
                            else if (associatedControl is DropDownList)
                            {
                                string selectedText = (associatedControl as DropDownList).SelectedItem.Text;
                                if (selectedText != string.Empty)
                                {
                                    sb.AppendFormat("<b>{0} :</b> {1}<br />", lbl.Text, (associatedControl as DropDownList).SelectedItem.Text);
                                }
                            }
                            else if (associatedControl is RadioButtonList)
                            {
                                sb.AppendFormat("<b>{0} :</b> {1}<br />", lbl.Text, (associatedControl as RadioButtonList).SelectedValue);
                            }
                            else if (associatedControl is CheckBoxList)
                            {
                                CheckBoxList cbList = (CheckBoxList)associatedControl;
                                string selectedValues = string.Empty;
                                foreach (ListItem item in cbList.Items)
                                {
                                    if (item.Selected) selectedValues += ", " + item.Value;
                                }
                                selectedValues = selectedValues.Remove(0, 2);

                                sb.AppendFormat("<b>{0} :</b> {1}<br />", lbl.Text, selectedValues);
                            }
                            else if (associatedControl is FileUpload)
                            {
                                FileUpload fu = (FileUpload)associatedControl;
                                if (fu.HasFile && fu.PostedFile != null && !string.IsNullOrEmpty(fu.FileName))
                                {
                                    mailAttachments.Add(new Attachment(fu.FileContent, fu.FileName, fu.PostedFile.ContentType));
                                    totalFileSize += fu.FileBytes.Length;
                                    sb.AppendFormat("<b>{0} :</b> {1}<br />", lbl.Text, "Ekte");
                                }
                            }
                        }
                    }
                }
            }*/

            if (totalFileSize > maxFileSize)
            {
                ResultMessage.InnerText = "Mesajınız iletilemedi! Eklenen dosyaların toplam boyutu 15 MB'yi aşmamalı.";
                ResultMessage.Visible = true;
                return;
            }

            string emailBody = formData.ToString();
            
            MailMessage mm = new MailMessage("\"" + FormSender + "\" <" + FormSender + ">", FormTo);
            mm.Subject = CommonMethods.StripHtmlTags(TitleHtml).Trim();
            if (mm.Subject == string.Empty)
            {
                mm.Subject = "Form : " + (Page as Esnek.BLL.Page).Title;
            }
            mm.IsBodyHtml = true;
            mm.Body = emailBody;
            if (mailAttachments.Count > 0)
            {
                foreach (Attachment att in mailAttachments)
                {
                    mm.Attachments.Add(att);
                }
            }

            try
            {
                (Page as Page).CurrentSite.SendMailMessage(mm);
                ResultMessage.InnerText = "Mesajınız başarıyla iletildi!";
                ResultMessage.Visible = true;
            }
            catch
            {
                ResultMessage.InnerText = "Hata! Mesajınız İletilemedi";
                ResultMessage.Visible = true;
            }

        }

        protected void Clear_Click(object sender, EventArgs e)
        {
            Response.Redirect((Page as Page).Url);
        }
    }

}