using System;
using System.Web;
using System.Data;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using SubSonic.Repository;
using SubSonic.DataProviders;
using SubSonic.Query;
using HtmlAgilityPack;
using System.Text.RegularExpressions;
using Esnek.BLL;
using Esnek.DAL;

namespace Esnek.BLL.PartTemplates
{

    [PartTemplate(Name = "Menü", TemplateName = "MenuWithZones", SortNo = 400)]
    public partial class MenuWithZones : PartWithZones, IMenu
    {

        [TitledTextBox(Title = "Başlık", IsRequired=true, SortNo=100)]
        public string Name
        {
            get { return GetPropertyValue<string>("Name"); }
            set { SetPropertyValue<string>("Name", value); }
        }

        [CheckBox(Title = "Başlık Görünsün", SortNo = 200)]
        public bool ShowTitle
        {
            get { return GetPropertyValue<bool>("ShowTitle"); }
            set { SetPropertyValue<bool>("ShowTitle", value); }
        }

        [PageUrlProperty(Title = "Kök Sayfa", SortNo = 300)]
        public string RootPage
        {
            get { return GetPropertyValue<string>("RootPage"); }
            set { SetPropertyValue<string>("RootPage", value); }
        }

        [TitledTextBox(Title = "Derinlik", IsRequired = true, SortNo = 400)]
        public string DepthLevel
        {
            get { return GetPropertyValue<string>("DepthLevel", "3"); }
            set { SetPropertyValue<string>("DepthLevel", value); }
        }

        [CheckBox(Title = "Dikey Görünüm", SortNo = 450)]
        public bool IsVertical
        {
            get { return GetPropertyValue<bool>("IsVertical"); }
            set { SetPropertyValue<bool>("IsVertical", value); }
        }

        [CheckBox(Title = "Eşit Genişlik", SortNo = 550, AuthorizedRoles = new string[] {"Super User"})]
        public bool EqualWidth
        {
            get { return GetPropertyValue<bool>("EqualWidth"); }
            set { SetPropertyValue<bool>("EqualWidth", value); }
        }

        [ControlFactory(Title = "Sayfalar", OutputTypes = new string[] { ControlFactoryOutputTypes.PagePathV2 }, SortNo = 200,
    CssClass = "factory-gallery", TabName=TabNames.Include)]
        public ControlFactory Include
        {
            get { return GetPropertyValue<ControlFactory>("Include"); }
            set { SetPropertyValue<ControlFactory>("Include", value); }
        }

        [ControlFactory(Title = "Sayfalar", OutputTypes = new string[] { ControlFactoryOutputTypes.PagePathV2 }, SortNo = 200,
    CssClass = "factory-gallery", TabName = TabNames.Exclude)]
        public ControlFactory Exclude
        {
            get { return GetPropertyValue<ControlFactory>("Exclude"); }
            set { SetPropertyValue<ControlFactory>("Exclude", value); }
        }

        [DBProperty]
        public override string ZoneID
        {
            get
            {
                string znID = GetPropertyValue<string>("ZoneID");
                return !string.IsNullOrWhiteSpace(znID) ? znID : ContentDataAccess.GetDefaultPartZoneMapping(PartType);
            }
            set { SetPropertyValue<string>("ZoneID", value); }
        }

        [DBProperty]
        public string Implements
        {
            get { return "IMenu"; }
        }

        ContentDataAccess DatabaseManager = new ContentDataAccess();

        Zone LevelN = null;
        
        int MaxLevel = 0;
        int depthLevel = 1;
        
        HashSet<string> ExcludedPages = new HashSet<string>();

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            
            if (!Page.ClientScript.IsClientScriptIncludeRegistered("menujs"))
            {
                Page.ClientScript.RegisterClientScriptInclude(Page.GetType(), "menujs", "/sites/default/js/jquery.hoverIntent.minified.js");
            }

            int.TryParse(DepthLevel, out depthLevel);

            ViewMode ViewMode;
            UserProfile UserProfile = (Page as Page).UserProfile;

            if (UserProfile != null)
                ViewMode = UserProfile.ViewMode;
            else
                ViewMode = ViewMode.Published;

            DataTable MenuItems = null;

            if (string.IsNullOrWhiteSpace(this.RootPage) && Include != null && Include.Controls.Count > 0)
            {
                HashSet<string> added = new HashSet<string>();

                foreach (PagePathV2 path in Include.Controls)
                {
                    if (added.Contains(path.PageUrl)) continue;
                    added.Add(path.PageUrl);

                    DataTable PageMenuItems = DatabaseManager.GetMenuItemsByRootPage(path.PageUrl, ViewMode, true, depthLevel);

                    if (PageMenuItems == null || PageMenuItems.Rows.Count == 0) continue;
                    else if (MenuItems == null) MenuItems = PageMenuItems;
                    else
                    {
                        MenuItems.Merge(PageMenuItems, false, MissingSchemaAction.Ignore);
                    }
                }

            }
            else
            {
                if (this.RootPage == "auto")
                {                    
                    this.RootPage = DatabaseManager.GetPagePath(DatabaseManager.GetZone(this.ContainerID).PageID);
                }
                MenuItems = DatabaseManager.GetMenuItemsByRootPage(this.RootPage, ViewMode, false, depthLevel);
            }


            if (Exclude != null && Exclude.Controls.Count > 0)
            {
                foreach (PagePathV2 path in Exclude.Controls)
                {
                    if (ExcludedPages.Contains(path.PageUrl)) continue;
                    ExcludedPages.Add(path.PageUrl);
                }
            }


            /*if (!IsSimpleMenu)
                MenuItems = DatabaseManager.GetMenuItems(this.ID, ViewMode);
            else
                MenuItems = DatabaseManager.GetMenuItemsByRootPage(this.RootPage, ViewMode);*/
           
            if (MenuItems == null) return;

            AppendPages(MenuItems, UserProfile, ViewMode, this.RootPage);

        }

        public void MarkAsConvertedZone(Zone zn)
        {
            MarkAsConvertedZone(zn, true);
        }

        public void MarkAsConvertedZone(Zone zn, bool deep)
        {
            if (deep && zn.ChildZones != null)
            {
                foreach (Zone child in zn.ChildZones)
                    MarkAsConvertedZone(child, deep);
            }

            string classname = " convertedzone ";
            if (zn.HasClft) classname += "clft ";

            zn.Container.Attributes["class"] += classname;

        }

        public void FillItem(ref Zone item, ref HtmlGenericControl anca, string PagePath, bool IsCurrentPage, bool IsRedirectPage, bool IsNewWindow, int i, DataRow[] TopPages, DataRow TopPage, bool IsActiveState)
        {
            if (!IsActiveState) item.Container.Attributes["class"] += " item-default ";
            else item.Container.Attributes["class"] += " item-active ";

            Zone ImageContainer = null;
            Zone LinkContainer = null;
            Zone TextContainer = null;

            LoadContainers(item, ref ImageContainer, ref LinkContainer, ref TextContainer);

            HtmlGenericControl anc = new HtmlGenericControl("a");
            anc.Attributes["href"] = PagePath;
            anc.Attributes["class"] += " link ";
            if (IsCurrentPage)
            {
                anc.Attributes["class"] += " current ";
                if (!IsActiveState)
                {
                    item.ParentZone.Container.Attributes["class"] += " current ";
                }
            }
            if (IsRedirectPage) anc.Attributes["class"] += " redirect ";
            if (IsNewWindow) anc.Attributes["target"] = "_blank";

            if (!IsActiveState)
            {
                if (i == 0)
                {
                    //anc.Attributes["class"] += "first ";
                    item.ParentZone.Container.Attributes["class"] += " first ";
                }
                
                if (i == TopPages.Length - 1)
                {
                    //anc.Attributes["class"] += "last ";
                    item.ParentZone.Container.Attributes["class"] += " last ";
                }
            }

            LinkContainer.Container = anc;

            MarkAsConvertedZone(item);

            string formatText = CommonMethods.StripHtmlTags(TextContainer.FormatText);
            if (string.IsNullOrWhiteSpace(formatText)) formatText = null;
            else formatText = TextContainer.FormatText;
            if (formatText != null)
            {
                MatchCollection matches = Regex.Matches(formatText, @"\[(.*?)\]");
                if (matches.Count == 0) formatText = null;
                else
                {
                    foreach (Match m in matches)
                    {
                        string mValue = m.Value.Trim(new char[] { '[', ']' });
                        object data = TopPage[mValue];
                        string val = data != DBNull.Value ? data.ToString() : null;
                        formatText = formatText.Replace(m.Value, val);
                    }
                }
            }

            if (ImageContainer != null)
            {
                string imageUrl = CommonMethods.StripHtmlTags(ImageContainer.FormatText);
                if (string.IsNullOrWhiteSpace(imageUrl)) formatText = null;
                else imageUrl = ImageContainer.FormatText;
                if (imageUrl != null)
                {
                    MatchCollection matches = Regex.Matches(imageUrl, @"\[(.*?)\]");
                    if (matches.Count == 0) imageUrl = null;
                    else
                    {
                        foreach (Match m in matches)
                        {
                            string mValue = m.Value.Trim(new char[] { '[', ']' });
                            object data = TopPage[mValue];
                            imageUrl = data != DBNull.Value ? data.ToString() : null;
                        }
                    }
                }
                if (imageUrl != null)
                {
                    /*HtmlGenericControl valigner = new HtmlGenericControl("span");
                    valigner.Attributes["class"] = "valigner";
                    ImageContainer.Container.Controls.Add(valigner);
                    HtmlImage img = new HtmlImage();
                    img.Src = imageUrl;
                    ImageContainer.Container.Controls.Add(img);*/
                    Literal lt = new Literal();
                    lt.Text = string.Format(@"<table style=""height:100%""><tr><td style=""height:100%; vertical-align:middle""><img src=""{0}"" /></td></tr></table>", imageUrl);
                    ImageContainer.Container.Controls.Add(lt);
                }

            }


            /*HtmlNode formatNode = null;
            HtmlNode textNode = null;

            if(CommonMethods.StripHtmlTags(TextContainer.FormatText) != string.Empty){

                HtmlDocument doc = new HtmlDocument();
                doc.LoadHtml(TextContainer.FormatText);
                Response.Write("<pre>" + TextContainer.FormatText + "</pre>");
                formatNode = doc.DocumentNode;
                textNode = formatNode;
                while (textNode.FirstChild != null && (textNode.FirstChild.NodeType == HtmlNodeType.Element))
                {
                    for (int j = 1; j < textNode.ChildNodes.Count; j++)
                    {
                        textNode.ChildNodes[j].Remove();
                        j--;
                    }
                    textNode = textNode.FirstChild;
                }

                if (textNode.Attributes["class"] == null)
                {
                    textNode.Attributes.Add("class", "text-node");
                }
                else textNode.Attributes["class"].Value += " text-node";
                    
            }

            if (TopPage["Text"] != DBNull.Value)
            {
                string extraText = TopPage["Text"].ToString();
                if (formatNode == null)
                {                        
                    Literal lt = new Literal();
                    lt.Text = "<span class=\"pagetitle\">" + pageTitle + "</span>" + extraText;
                    TextContainer.Container.Controls.Add(lt);
                }
                else
                {
                    textNode.InnerHtml = "<span class=\"pagetitle\">" + pageTitle + "</span>" + extraText;
                    TextContainer.Container.InnerHtml = formatNode.OuterHtml;
                }
            }
            else
            {
                if (formatNode == null)
                {
                    Literal lt = new Literal();
                    lt.Text = pageTitle;
                    TextContainer.Container.Controls.Add(lt);
                }
                else
                {
                    textNode.InnerHtml = pageTitle;
                    TextContainer.Container.InnerHtml = formatNode.OuterHtml;
                }
            }*/

            TextContainer.Container.Attributes["class"] += " text ";

            if (formatText != null)
            {
                TextContainer.Container.InnerHtml = formatText;
            }
            else
            {
                string pageTitle = TopPage["Title"].ToString();
                TextContainer.Container.InnerHtml = pageTitle;
            }

        }

        private void AppendPages(DataTable MenuItems, UserProfile UserProfile, ViewMode ViewMode, string RootPage)
        {
            DataRow[] TopPages = null;

            string IDList = string.Empty;

            for (int i = 0; i < MenuItems.Rows.Count; i++)
            {
                DataRow dr = MenuItems.Rows[i];
                if (dr["AuthorizedRoles"] != DBNull.Value)
                {
                    string authorizedRolesString = dr["AuthorizedRoles"].ToString();
                    if (authorizedRolesString == "?")
                    {
                        continue;
                    }
                    else
                    {
                        string[] roles = authorizedRolesString.Split('$');
                        bool isauthorized = false;
                        foreach (string role in roles)
                        {
                            if (UserProfile.Roles.ContainsKey(role))
                            {
                                isauthorized = true;
                                break;
                            }
                        }

                        if (!isauthorized)
                        {
                            MenuItems.Rows.RemoveAt(i);
                            i--;
                        }
                    }
                }
            }

            if (string.IsNullOrWhiteSpace(this.RootPage) && Include != null && Include.Controls.Count > 0)
            {
                HashSet<string> added = new HashSet<string>();
                foreach (PagePathV2 path in Include.Controls)
                {
                    if (added.Contains(path.PageUrl)) continue;
                    added.Add(path.PageUrl);

                    DataRow[] PageTopPages; 

                    if (string.IsNullOrWhiteSpace(path.PageUrl)) path.PageUrl = "/";

                    //mevlut bad code
                    if (path.PageUrl.EndsWith(".aspx")) path.PageUrl = path.PageUrl.Substring(0, path.PageUrl.Length - 5);

                    PageTopPages = MenuItems.Select(string.Format("Path LIKE '{0}'", path.PageUrl));

                    if (PageTopPages == null || PageTopPages.Length == 0) continue;
                    else if (TopPages == null) TopPages = PageTopPages;
                    else
                    {
                        DataRow[] first = TopPages;
                        DataRow[] second = PageTopPages;
                        TopPages = new DataRow[first.Length + second.Length];
                        Array.Copy(first, TopPages, first.Length);
                        Array.Copy(second, 0, TopPages, first.Length, second.Length);                      
                    }
                }
            }

            if (TopPages == null)
            {
                if (RootPage == null || RootPage == string.Empty) RootPage = "/";

                if (RootPage == "/")
                    TopPages = MenuItems.Select(string.Format("ID = OwnerID"));
                else
                {
                    //mevlut bad code
                    if (RootPage.EndsWith(".aspx")) RootPage = RootPage.Substring(0, RootPage.Length - 5);

                    DataRow pgs = DatabaseManager.GetPageByPath(RootPage, ViewMode);
                    if (pgs == null) return;
                    string RootPageID = pgs["ID"].ToString();

                    TopPages = MenuItems.Select(string.Format("OwnerID = '{0}' and ID <> '{0}'", RootPageID));
                }
            }

            List<DataRow> sorter = new List<DataRow>(TopPages);
            sorter.Sort(delegate(DataRow a, DataRow b)
            {
                int sortNo_a = 0;
                int sortNo_b = 0;
                sortNo_a = Convert.ToInt32(a["SortNo"].ToString());
                sortNo_b = Convert.ToInt32(b["SortNo"].ToString());
                return sortNo_a.CompareTo(sortNo_b);
            });
            TopPages = sorter.ToArray();

            Zone convertedContainer = DatabaseManager.GetZone(ZoneID, true, true, ViewMode.Preview);

            if (convertedContainer == null) return;

            convertedContainer.Width.Value = "width:100%";

            for (int i = 0; i < convertedContainer.CssStyles.Controls.Count; i++)
            {
                CssStyles cssStyle = (CssStyles)convertedContainer.CssStyles.Controls[i];
                if (cssStyle.StyleName.StartsWith("margin"))
                {
                    convertedContainer.CssStyles.Controls.RemoveAt(i--);
                }
            }

            CreateSymmetry(ref convertedContainer);

            Zone ItemContainer = convertedContainer;

            Zone menu = CommonMethods.CloneZone(ItemContainer);

            menu.MinHeight.Value = "min-height:0px";

            MarkAsConvertedZone(menu);

            int c = 0;

            Zone currentLevel = null;

            LoadItemContainer(menu, ref currentLevel, "level-" + c.ToString());

            LevelN = CommonMethods.CloneZone(currentLevel);

            if (currentLevel == null) return;

            Zone itemStatesSource = null;

            LoadItemContainer(currentLevel, ref itemStatesSource, "itemstates-" + c.ToString());

            AdjustMargins(ref itemStatesSource);

            for (int i = 0; i < TopPages.Length; i++)
            {
                DataRow TopPage = TopPages[i];

                string PagePath = TopPage["Path"] + ".aspx";

                if (ExcludedPages.Contains(PagePath)) continue;

                bool IsCurrentPage = Page.ID == TopPage["ID"].ToString();

                bool IsRedirectPage = false;

                bool IsNewWindow = false;

                if (!IsCurrentPage)
                {
                    IsRedirectPage = TopPage["TemplateName"].ToString() == "Redirect";

                    if (IsRedirectPage)
                    {
                        IsNewWindow = (TopPage["NewWindow"] != DBNull.Value && TopPage["NewWindow"].ToString() == "True");

                        IsCurrentPage = Page.ID == DatabaseManager.GetPageID(TopPage["RedirectUrl"].ToString());

                    }
                }

                Zone itemStates = null;
                Zone itemDefault = null;
                Zone itemActive = null;

                itemStates = CommonMethods.CloneZone(itemStatesSource);
                itemStates.Container.Attributes["class"] += " item ";
                MarkAsConvertedZone(itemStates);

                currentLevel.ChildZones.Add(itemStates);

                LoadItemContainer(itemStates, ref itemDefault, "item-" + c.ToString());
                LoadItemContainer(itemStates, ref itemActive, "item-" + c.ToString() + "-active");

                HtmlGenericControl anc = new HtmlGenericControl("a");

                FillItem(ref itemDefault, ref anc, PagePath, IsCurrentPage, IsRedirectPage, IsNewWindow, i, TopPages, TopPage, false);

                List<HtmlGenericControl> itemQueue = new List<HtmlGenericControl>();
                itemQueue.Add(anc);

                anc = new HtmlGenericControl("a");

                FillItem(ref itemActive, ref anc, PagePath, IsCurrentPage, IsRedirectPage, IsNewWindow, i, TopPages, TopPage, true);

                AppendSubPages(c, itemStates, TopPage, MenuItems, itemQueue);
            }

            currentLevel.Container.Attributes["class"] += " lvl lvl" + c.ToString() + " ";

            if(itemStatesSource != null)
                itemStatesSource.ParentZone.ChildZones.Remove(itemStatesSource);

            phlMenu.Controls.Add(menu);

        }

        private void AppendSubPages(int level, Zone Container, DataRow MenuItem, DataTable MenuItems, List<HtmlGenericControl> ItemQueue)
        {
            level++;
            
            if (level >= depthLevel) return;

            Zone currentLevel = null;

            LoadItemContainer(Container, ref currentLevel, "level-" + level.ToString());

            DataRow[] SubPages;

            SubPages = MenuItems.Select(string.Format("ID <> {0} and OwnerID = {0}", MenuItem["ID"].ToString()));

            if (SubPages.Length == 0)
            {
                if (currentLevel != null && currentLevel.ParentZone != null) currentLevel.ParentZone.ChildZones.Remove(currentLevel);
                return;
            }

            if (currentLevel == null)
            {
                currentLevel = CommonMethods.CloneZone(LevelN);                
                MarkAsConvertedZone(currentLevel);
                Container.ChildZones.Add(currentLevel);
                currentLevel.ParentZone = Container;

                if(MaxLevel == 0) MaxLevel = level;
            }
            else 
                LevelN = CommonMethods.CloneZone(currentLevel);

            Zone itemStatesSource = null;

            string checkLevel = MaxLevel == 0 ? level.ToString() : (MaxLevel - 1).ToString();

            LoadItemContainer(currentLevel, ref itemStatesSource, "itemstates-" + checkLevel);

            AdjustMargins(ref itemStatesSource);

            if (SubPages.Length > 0)
            {
                //mevlut bad code
                List<DataRow> sorter = new List<DataRow>(SubPages);
                sorter.Sort(delegate(DataRow a, DataRow b)
                {
                    int sortNo_a = 0;
                    int sortNo_b = 0;
                    sortNo_a = Convert.ToInt32(a["SortNo"].ToString());
                    sortNo_b = Convert.ToInt32(b["SortNo"].ToString());
                    return sortNo_a.CompareTo(sortNo_b);
                });
                SubPages = sorter.ToArray();
                //bad code end

                for (int i = 0; i < SubPages.Length; i++)
                {
                    DataRow SubPage = SubPages[i];

                    string PagePath = SubPage["Path"] + ".aspx";

                    if (ExcludedPages.Contains(PagePath)) continue;

                    bool IsCurrentPage = Page.ID == SubPage["ID"].ToString();

                    bool IsRedirectPage = false;

                    bool IsNewWindow = false;

                    if (!IsCurrentPage)
                    {
                        IsRedirectPage = SubPage["TemplateName"].ToString() == "Redirect";

                        if (IsRedirectPage)
                        {
                            IsNewWindow = (SubPage["NewWindow"] != DBNull.Value && SubPage["NewWindow"].ToString() == "True");

                            IsCurrentPage = Page.ID == DatabaseManager.GetPageID(SubPage["RedirectUrl"].ToString());

                        }
                    }

                    Zone itemStates = null;
                    Zone itemDefault = null;
                    Zone itemActive = null;

                    itemStates = CommonMethods.CloneZone(itemStatesSource);
                    itemStates.Container.Attributes["class"] += " item ";
                    MarkAsConvertedZone(itemStates);

                    currentLevel.ChildZones.Add(itemStates);

                    LoadItemContainer(itemStates, ref itemDefault, "item-" + checkLevel.ToString());
                    LoadItemContainer(itemStates, ref itemActive, "item-" + checkLevel.ToString() + "-active");

                    HtmlGenericControl anc = new HtmlGenericControl("a");

                    FillItem(ref itemDefault, ref anc, PagePath, IsCurrentPage, IsRedirectPage, IsNewWindow, i, SubPages, SubPage, false);

                    List<HtmlGenericControl> newItemQueue = new List<HtmlGenericControl>(ItemQueue);
                    newItemQueue.Add(anc);

                    anc = new HtmlGenericControl("a");

                    FillItem(ref itemActive, ref anc, PagePath, IsCurrentPage, IsRedirectPage, IsNewWindow, i, SubPages, SubPage, true);

                    AppendSubPages(level, itemStates, SubPage, MenuItems, newItemQueue);
                }

                currentLevel.Container.Attributes["class"] += " lvl lvl" + level.ToString() + " ";
            }

            if (itemStatesSource != null)
                itemStatesSource.ParentZone.ChildZones.Remove(itemStatesSource);
        }

        private void AdjustMargins(ref Zone itemStatesSource)
        {
            Zone itemSource = null;
            Zone itemCurrentSource = null;
            Zone levelSource = null;

            foreach (Zone zn in itemStatesSource.ChildZones)
            {
                if (zn.PartRole.StartsWith("item-"))
                {
                    if (!zn.PartRole.EndsWith("-active")) itemSource = zn;
                    else itemCurrentSource = zn;
                }
                else if (zn.PartRole.StartsWith("level-")) levelSource = zn;
            }

            if (itemSource != null && itemCurrentSource != null)
            {
                    Zone first = null;
                    Zone second = null;

                    if (itemSource.ParentZone.ChildZones.IndexOf(itemSource) < itemCurrentSource.ParentZone.ChildZones.IndexOf(itemCurrentSource))
                    {
                        first = itemSource;
                        second = itemCurrentSource;
                    }
                    else
                    {
                        second = itemSource;
                        first = itemCurrentSource;
                    }

                    System.Globalization.CultureInfo ci = new System.Globalization.CultureInfo("en-GB");

                    if (!second.HasClft)
                    {
                        CssStyles marginRight = GetStyle(first, "margin-right");
                        double mR = 0;
                        if(marginRight != null){
                            double.TryParse(marginRight.StyleValue.Replace("px", ""), System.Globalization.NumberStyles.AllowDecimalPoint, ci, out mR);
                        }

                        CssStyles marginLeft = GetStyle(second, "margin-left");
                        double mL = 0;
                        if (marginLeft != null)
                        {
                            double.TryParse(marginLeft.StyleValue.Replace("px", ""), System.Globalization.NumberStyles.AllowDecimalPoint, ci, out mL);
                        }

                        mR = mL = (mR + mL) / (double)2;

                        marginLeft = new CssStyles();
                        marginLeft.StyleName = "margin-left";
                        marginLeft.StyleValue = mL.ToString("#,0.####", ci) + "px";

                        marginRight = new CssStyles();
                        marginRight.StyleName = "margin-right";
                        marginRight.StyleValue = mR.ToString("#,0.####", ci) + "px";

                        SetStyle(first, new CssStyles[] { marginLeft, marginRight });
                        SetStyle(second, new CssStyles[] { marginLeft, marginRight });
                        if (levelSource != null) SetStyle(levelSource, new CssStyles[] { marginLeft, marginRight });
                    }            
                    else
                    {
                        if (itemSource.PartRole == "item-0") IsVertical = true;

                        CssStyles marginBottom = GetStyle(first, "margin-bottom");
                        double mB = 0;
                        if (marginBottom != null)
                        {
                            double.TryParse(marginBottom.StyleValue.Replace("px", ""), System.Globalization.NumberStyles.AllowDecimalPoint, ci, out mB);
                        }

                        CssStyles marginTop = GetStyle(second, "margin-top");
                        double mT = 0;
                        if (marginTop != null)
                        {
                            double.TryParse(marginTop.StyleValue.Replace("px", ""), System.Globalization.NumberStyles.AllowDecimalPoint, ci, out mT);
                        }

                        mT = mB = (mT + mB) / (double)2;

                        marginTop = new CssStyles();
                        marginTop.StyleName = "margin-top";
                        marginTop.StyleValue = mT.ToString("#,0.####", ci) + "px";

                        marginBottom = new CssStyles();
                        marginBottom.StyleName = "margin-bottom";
                        marginBottom.StyleValue = mB.ToString("#,0.####", ci) + "px";

                        /*mB = mT + mB;
                        mT = 0;

                        marginTop = new CssStyles();
                        marginTop.StyleName = "margin-top";
                        marginTop.StyleValue = mT.ToString("#,0.####", ci) + "px";

                        marginBottom = new CssStyles();
                        marginBottom.StyleName = "margin-bottom";
                        marginBottom.StyleValue = mB.ToString("#,0.####", ci) + "px";*/

                        SetStyle(first, new CssStyles[] { marginTop, marginBottom });
                        SetStyle(second, new CssStyles[] { marginTop, marginBottom });
                    }


                    int minH1 = 0;
                    int minH2 = 0;

                    int.TryParse(first.MinHeight.Value.Split(':')[1].Replace("px", ""), out minH1);
                    int.TryParse(second.MinHeight.Value.Split(':')[1].Replace("px", ""), out minH2);

                    int minH = Math.Max(minH1, minH2);

                    first.MinHeight.Value = "min-height:" + minH.ToString() + "px";
                    second.MinHeight.Value = "min-height:" + minH.ToString() + "px";
            }
        }

        public void LoadContainers(Zone container, ref Zone ImageContainer, ref Zone LinkContainer, ref Zone TextContainer)
        {
            foreach (Zone zn in container.ChildZones)
            {
                //zn.Container.Attributes["class"] += " convertedzone ";

                if (zn.PartRole == "image")
                {
                    if (ImageContainer == null)
                        ImageContainer = zn;
                }
                else if (zn.PartRole == "link")
                {
                    if (LinkContainer == null)
                        LinkContainer = zn;
                }
                else if (zn.PartRole == "text")
                {
                    if (TextContainer == null)
                        TextContainer = zn;
                }

                if (zn.ChildZones != null)
                {
                    LoadContainers(zn, ref ImageContainer, ref LinkContainer, ref TextContainer);
                }
            }
        }

        public void LoadItemContainer(Zone container, ref Zone ItemContainer, string levelName)
        {
            if (container.ChildZones == null) return;

            foreach (Zone zn in container.ChildZones)
            {
                if (zn.PartRole == levelName)
                {
                    ItemContainer = zn;
                    break;
                }

                if (zn.ChildZones != null && ItemContainer == null)
                {
                    LoadItemContainer(zn, ref ItemContainer, levelName);
                }
            }
        }

        Dictionary<int, Zone> Levels = new Dictionary<int, Zone>();

        Dictionary<int, Zone> ItemStatesDict = new Dictionary<int, Zone>();

        public void CreateSymmetry(ref Zone PartTemplate)
        {
            bool rmoved = false;

            LoadLevels(PartTemplate, out rmoved, 0);

            PartTemplate.ChildZones.Add(Levels[0]);
            Levels.Remove(0);

            AddLevels(PartTemplate, 0);
        }

        public void AddLevels(Zone zone, int level)
        {
            Zone levelzone = null;

            if (zone.PartRole != null && zone.PartRole.StartsWith("item"))
            {
                level = Convert.ToInt32(zone.PartRole.Split('-')[1]);
            }

            if (zone.PartRole != null && zone.PartRole.StartsWith("item-" + level.ToString()))
            {
                Zone itemStates = null;
                if (!ItemStatesDict.ContainsKey(level))
                {
                    itemStates = new Zone();
                    itemStates.PartRole = "itemstates-" + level.ToString();
                    ItemStatesDict.Add(level, itemStates);
                    zone.ParentZone.ChildZones.Insert(zone.ParentZone.ChildZones.IndexOf(zone), itemStates);
                    itemStates.ParentZone = zone.ParentZone;
                    itemStates.ChildZones = new List<Zone>();
                    itemStates.ChildZones.Add(zone);
                    zone.ParentZone.ChildZones.Remove(zone);
                    zone.ParentZone = itemStates;

                    if (Levels.ContainsKey(level + 1))
                    {
                        levelzone = CommonMethods.CloneZone(Levels[level + 1]);
                        MarkAsConvertedZone(levelzone);
                        levelzone.SortNo = 999999;
                        if (levelzone.ParentZone != null) levelzone.ParentZone.ChildZones.Remove(levelzone);
                        itemStates.ChildZones.Add(levelzone);
                        levelzone.ParentZone = itemStates;
                    }

                }
                else
                {
                    itemStates = ItemStatesDict[level];
                    itemStates.ChildZones.Add(zone);
                    zone.ParentZone.ChildZones.Remove(zone);
                    zone.ParentZone = itemStates;
                }
            }


            if (levelzone != null) zone = levelzone;

            if (zone.ChildZones != null)
            {
                for (int i = 0; i < zone.ChildZones.Count; i++)
                {
                    Zone child = zone.ChildZones[i];
                    AddLevels(child, level);
                }
            }

        }

        public void LoadLevels(Zone zone, out bool removed, int level)
        {
            removed = false;

            if (zone.PartRole != null && zone.PartRole.StartsWith("level"))
            {
                int zonelevel = Convert.ToInt32(zone.PartRole.Split('-')[1]);
                level = zonelevel;

                if (!Levels.ContainsKey(zonelevel))
                {
                    Levels[zonelevel] = zone;
                    zone.ParentZone.ChildZones.Remove(zone);
                    removed = true;
                }

                if(!IsVertical) RemoveStyle(zone, "margin*");
                zone.MinHeight.Value = "min-height:0";
            }            

            if (!IsVertical)
            {
                if (!EqualWidth && level == 0 && ((zone.PartRole != null && (zone.PartRole.StartsWith("level") || zone.PartRole.StartsWith("item") || zone.PartRole.StartsWith("link") || zone.PartRole.StartsWith("text"))) || HasTextZone(zone)))
                {
                    zone.Width.Value = "width:auto";
                }           
            }

            if (zone.ChildZones != null)
            {
                for (int i = 0; i < zone.ChildZones.Count; i++)
                {
                    Zone child = zone.ChildZones[i];
                    bool rmoved = false;
                    LoadLevels(child, out rmoved, level);
                    if (rmoved) i--;
                }
            }
        }

        public bool HasTextZone(Zone zone)
        {
            if (zone.PartRole == "text") return true;
            if (zone.ChildZones != null)
            {
                foreach (Zone child in zone.ChildZones)
                {
                    if (child.PartRole == "text") return true;
                    else if (HasTextZone(child)) return true;
                }
            }
            return false;
        }

        public void RemoveStyle(Zone zone, string styleName)
        {
            if (zone.CssStyles == null) return;

            bool startsWith = false;

            if (styleName.EndsWith("*")) { startsWith = true; styleName = styleName.TrimEnd('*'); }

            for (int i = 0; i < zone.CssStyles.Controls.Count; i++)
            {
                CssStyles cssStyle = (CssStyles)zone.CssStyles.Controls[i];
                if ((!startsWith && cssStyle.StyleName == styleName) || (startsWith && cssStyle.StyleName.StartsWith(styleName)))
                {
                    zone.CssStyles.Controls.RemoveAt(i--);
                }
            }

        }

        public CssStyles GetStyle(Zone zone, string styleName)
        {
            if (zone.CssStyles == null) return null;

            CssStyles style = null;

            for (int i = 0; i < zone.CssStyles.Controls.Count; i++)
            {
                CssStyles cssStyle = (CssStyles)zone.CssStyles.Controls[i];
                if (cssStyle.StyleName == styleName)
                {
                    style = cssStyle;
                }
            }

            return style;
        }

        public void SetStyle(Zone zone, CssStyles[] styles)
        {
            if (zone.CssStyles == null) return;

            HashSet<CssStyles> stylesHash = new HashSet<CssStyles>(styles);

            for (int i = 0; i < zone.CssStyles.Controls.Count; i++)
            {
                CssStyles cssStyle = (CssStyles)zone.CssStyles.Controls[i];

                foreach (CssStyles style in styles)
                {
                    if (cssStyle.StyleName == style.StyleName)
                    {
                        cssStyle.StyleValue = style.StyleValue;
                        if (stylesHash.Contains(style))
                        {
                            stylesHash.Remove(style);
                        }
                    }
                }                
            }

            foreach (CssStyles cssStyle in stylesHash)
            {
                CssStyles style = new CssStyles();
                style.StyleValue = cssStyle.StyleValue;
                style.StyleName = cssStyle.StyleName;
                zone.CssStyles.Controls.Add(style);
            }

        }
    }
}