﻿using System;
using System.Web;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Web.UI;
using System.Collections.Generic;
using Esnek.BLL;
using Esnek.BLL.WebControls;
using Esnek.DAL;
using System.Data;
using SubSonic;
using SubSonic.Repository;
using SubSonic.Query;
using SubSonic.DataProviders;

namespace Esnek.BLL.PartTemplates
{
    [PartTemplate(Name = "Bölümlenmiş İçerik", TemplateName = "TextsWithZones", SortNo = 500)]
    public partial class TextsWithZones : PartWithZones
    {

        [TitledTextBox(Title = "Başlık :", SortNo = 100)]
        public string Title
        {
            get { return GetPropertyValue<string>("Title"); }
            set { SetPropertyValue<string>("Title", value); }
        }

        [ControlFactory(Title = "İçerikler", OutputTypes = new string[] { ControlFactoryOutputTypes.ContentItem },
            CssClass = "factory-gallery", SortNo = 200)]
        public ControlFactory Contents
        {
            get { return GetPropertyValue<ControlFactory>("Contents"); }
            set { SetPropertyValue<ControlFactory>("Contents", value); }
        }

        [DBProperty]
        public override string ZoneID
        {
            get
            {
                string znID = GetPropertyValue<string>("ZoneID");
                return !string.IsNullOrWhiteSpace(znID) ? znID : ContentDataAccess.GetDefaultPartZoneMapping(PartType);
            }
            set { SetPropertyValue<string>("ZoneID", value); }
        }

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            if (Contents == null || Contents.Controls == null || Contents.Controls.Count == 0) return;

            ContentDataAccess DatabaseManager = new ContentDataAccess();

            Zone convertedContainer = DatabaseManager.GetZone(ZoneID, true, true, ViewMode.Preview);

            if (convertedContainer == null) return;

            for (int i = 0; i < convertedContainer.CssStyles.Controls.Count; i++)
            {
                CssStyles cssStyle = (CssStyles)convertedContainer.CssStyles.Controls[i];
                if (cssStyle.StyleName.StartsWith("margin"))
                {
                    convertedContainer.CssStyles.Controls.RemoveAt(i--);
                }
            }

            convertedContainer.Container.Attributes["class"] += " convertedzone ";

            SortChildZones(convertedContainer);

            MarkAsConvertedZone(convertedContainer);

            phlTextsWithZones.Controls.Add(convertedContainer);


            List<ContentItem> sorted = new List<ContentItem>();

            for (int i = 0; i < Contents.Controls.Count; i++)
            {
                sorted.Add((ContentItem)Contents.Controls[i]);
            }

            sorted.Sort(delegate(ContentItem a, ContentItem b)
            {
                return a.SortNo.CompareTo(b.SortNo);
            });

            for (int i = 0; i < sorted.Count; i++)
            {
                ContentItem content = (ContentItem)sorted[i];
                Zone contentContainer = null;
                LoadContainer(convertedContainer, ref contentContainer, "content-" + (i + 1).ToString());

                if (contentContainer != null)
                {
                    if (CommonMethods.StripHtmlTags(content.Content) != string.Empty)
                        contentContainer.Container.InnerHtml = content.Content;
                    else contentContainer.Parent.Controls.Remove(contentContainer);
                }
            }
        }

        public void SortChildZones(Zone zn)
        {
            if (zn.ChildZones != null)
            {
                foreach (Zone child in zn.ChildZones)
                    SortChildZones(child);

                zn.ChildZones.Sort(delegate(Zone a, Zone b)
                {
                    return a.SortNo.CompareTo(b.SortNo);
                });
            }

        }

        public void MarkAsConvertedZone(Zone zn)
        {
            if (zn.ChildZones != null)
            {
                foreach (Zone child in zn.ChildZones)
                    MarkAsConvertedZone(child);
            }

            zn.Container.Attributes["class"] += " convertedzone ";
        }

        public void LoadContainer(Zone container, ref Zone ItemContainer, string partRole)
        {
            foreach (Zone zn in container.ChildZones)
            {
                if (zn.PartRole == partRole)
                {
                    ItemContainer = zn;
                    break;
                }

                if (zn.ChildZones != null && ItemContainer == null)
                {
                    LoadContainer(zn, ref ItemContainer, partRole);
                }
            }
        }

        public bool RemoveBlock(Zone container, string blockName)
        {
            if (container.ChildZones == null) return false;

            for (int i = 0; i < container.ChildZones.Count; i++)
            {
                Zone childZone = container.ChildZones[i];
                if (childZone.PartRole == blockName)
                {
                    container.ChildZones.RemoveAt(i);
                    return true;
                }
                else if (RemoveBlock(childZone, blockName))
                {
                    return true;
                }
            }
            return false;
        }
    }
}
